@@ -1,6 +1,6 @@
 /*
  * asc85enc.c
- * Copyright (C) 2000,2001 A.J. van Os; Released under GPL
+ * Copyright (C) 2000-2003 A.J. van Os; Released under GPL
  *
  * Description:
  * Functions to for ASCII 85 encoding
@@ -72,7 +72,7 @@ vASCII85EncodeByte(FILE *pOutFile, int iByte)
 
 	if (iByte == EOF) {
 		/* End Of File, time to clean up */
-		if (iInBuffer > 0) {
+		if (iInBuffer > 0 && iInBuffer < 4) {
 			/* Encode the remaining bytes */
 			ulValue = 0;
 			for (iIndex = iInBuffer - 1; iIndex >= 0; iIndex--) {
@@ -120,17 +120,17 @@ vASCII85EncodeByte(FILE *pOutFile, int iByte)
  * vASCII85EncodeArray - ASCII 85 encode a byte array
  */
 void
-vASCII85EncodeArray(FILE *pInFile, FILE *pOutFile, int iLength)
+vASCII85EncodeArray(FILE *pInFile, FILE *pOutFile, size_t tLength)
 {
-	int	iCount, iByte;
+	size_t	tCount;
+	int	iByte;
 
 	fail(pInFile == NULL);
 	fail(pOutFile == NULL);
-	fail(iLength < 0);
 
-	DBG_DEC(iLength);
+	DBG_DEC(tLength);
 
-	for (iCount = 0; iCount < iLength; iCount++) {
+	for (tCount = 0; tCount < tLength; tCount++) {
 		iByte = iNextByte(pInFile);
 		if (iByte == EOF) {
 			break;
@@ -143,12 +143,12 @@ vASCII85EncodeArray(FILE *pInFile, FILE *pOutFile, int iLength)
  * vASCII85EncodeFile - ASCII 85 encode part of a file
  */
 void
-vASCII85EncodeFile(FILE *pInFile, FILE *pOutFile, int iLength)
+vASCII85EncodeFile(FILE *pInFile, FILE *pOutFile, size_t tLength)
 {
 	fail(pInFile == NULL);
 	fail(pOutFile == NULL);
-	fail(iLength <= 0);
+	fail(tLength == 0);
 
-	vASCII85EncodeArray(pInFile, pOutFile, iLength);
+	vASCII85EncodeArray(pInFile, pOutFile, tLength);
 	vASCII85EncodeByte(pOutFile, EOF);
 } /* end of vASCII85EncodeFile */
