@@ -1,6 +1,6 @@
 /*
  * depot.c
- * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
  *
  * Description:
  * Functions to compute the depot offset
@@ -86,8 +86,8 @@ bCreateSmallBlockList(ULONG ulStartblock, const ULONG *aulBBD, size_t tBBDLen)
 ULONG
 ulDepotOffset(ULONG ulIndex, size_t tBlockSize)
 {
+	ULONG	ulTmp;
 	size_t	tTmp;
-	int	iTmp;
 
 	fail(ulIndex >= ULONG_MAX / BIG_BLOCK_SIZE);
 
@@ -96,7 +96,7 @@ ulDepotOffset(ULONG ulIndex, size_t tBlockSize)
 		return (ulIndex + 1) * BIG_BLOCK_SIZE;
 	case SMALL_BLOCK_SIZE:
 		tTmp = (size_t)(ulIndex / SIZE_RATIO);
-		iTmp = (int)(ulIndex % SIZE_RATIO);
+		ulTmp = ulIndex % SIZE_RATIO;
 		if (aulSmallBlockList == NULL ||
 		    tTmp >= tSmallBlockListLen) {
 			DBG_HEX(aulSmallBlockList);
@@ -105,7 +105,7 @@ ulDepotOffset(ULONG ulIndex, size_t tBlockSize)
 			return 0;
 		}
 		return ((aulSmallBlockList[tTmp] + 1) * SIZE_RATIO +
-				iTmp) * SMALL_BLOCK_SIZE;
+				ulTmp) * SMALL_BLOCK_SIZE;
 	default:
 		DBG_DEC(tBlockSize);
 		DBG_FIXME();
