@@ -1,6 +1,6 @@
 /*
  * dib2sprt.c
- * Copyright (C) 2000,2001 A.J. van Os; Released under GPL
+ * Copyright (C) 2000-2003 A.J. van Os; Released under GPL
  *
  * Description:
  * Functions to translate dib pictures into sprites
@@ -8,7 +8,8 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "wimpt.h"
+#include "DeskLib:Error.h"
+#include "DeskLib:Sprite.h"
 #include "antiword.h"
 
 #if 0 /* defined(DEBUG) */
@@ -22,7 +23,7 @@ static int iPicCounter = 0;
 static int
 iGetByteWidth(const imagedata_type *pImg)
 {
-	switch (pImg->iBitsPerComponent) {
+	switch (pImg->uiBitsPerComponent) {
 	case  1:
 		return (pImg->iWidth + 31) / 32 * sizeof(int);
 	case  4:
@@ -31,70 +32,72 @@ iGetByteWidth(const imagedata_type *pImg)
 	case 24:
 		return (pImg->iWidth + 3) / 4 * sizeof(int);
 	default:
-		DBG_DEC(pImg->iBitsPerComponent);
+		DBG_DEC(pImg->uiBitsPerComponent);
 		return 0;
 	}
 } /* end of iGetByteWidth */
 
 /*
- * bCreateBlankSprite - Create a blank sprite.
+ * pCreateBlankSprite - Create a blank sprite.
  *
  * Create a blank sprite and add a palette if needed
  *
  * returns a pointer to the sprite when successful, otherwise NULL
  */
-static unsigned char *
-pucCreateBlankSprite(const imagedata_type *pImg, size_t *ptSize)
+static sprite_areainfo *
+pCreateBlankSprite(const imagedata_type *pImg, size_t *pSize)
 {
-	sprite_area	*pArea;
-	unsigned char	*pucTmp;
+	sprite_areainfo	*pArea;
+	UCHAR	*pucTmp;
 	size_t	tSize;
-	int	iIndex, iMode, iPaletteEntries, iByteWidth;
+	screen_modeval	uMode;
+	int	iIndex, iPaletteEntries;
 
-	DBG_MSG("pCreateBlankSprite");
+	TRACE_MSG("pCreateBlankSprite");
 
 	fail(pImg == NULL);
+	fail(pSize == NULL);
 
-	switch (pImg->iBitsPerComponent) {
+	switch (pImg->uiBitsPerComponent) {
 	case  1:
-		iMode = 18;
+		uMode.screen_mode = 18;
 		iPaletteEntries = 2;
 		break;
 	case  4:
-		iMode = 20;
+		uMode.screen_mode = 20;
 		iPaletteEntries = 16;
 		break;
 	case  8:
 	case 24:
-		iMode = 21;
+		uMode.screen_mode = 21;
 		iPaletteEntries = 0;
 		break;
 	default:
-		DBG_DEC(pImg->iBitsPerComponent);
+		DBG_DEC(pImg->uiBitsPerComponent);
 		return NULL;
 	}
 	fail(iPaletteEntries < 0 || iPaletteEntries > 16);
 
-	/* Create the sprite */
-
-	iByteWidth = iGetByteWidth(pImg);
-
-	tSize = sizeof(sprite_area) +
-		sizeof(sprite_header) +
-		iPaletteEntries * 8 +
-		iByteWidth * pImg->iHeight;
+	/* Get memory for the sprite */
+	tSize = sizeof(sprite_areainfo) +
+		Sprite_MemorySize(pImg->iWidth, pImg->iHeight, uMode,
+		iPaletteEntries > 0 ? sprite_HASPAL : sprite_HASNOMASKPAL);
 	DBG_DEC(tSize);
 	pArea = xmalloc(tSize);
-	sprite_area_initialise(pArea, tSize);
 
-	wimpt_noerr(sprite_create(pArea, "wordimage",
-		iPaletteEntries > 0 ? sprite_haspalette : sprite_nopalette,
-		pImg->iWidth, pImg->iHeight, iMode));
+	/* Initialise sprite area */
+	pArea->areasize = tSize;
+	pArea->numsprites = 0;
+	pArea->firstoffset = sizeof(sprite_areainfo);
+	pArea->freeoffset = sizeof(sprite_areainfo);
+
+	/* Create a blank sprite */
+	Error_CheckFatal(Sprite_Create(pArea, "wordimage",
+		iPaletteEntries > 0 ? 1 : 0,
+		pImg->iWidth, pImg->iHeight, uMode));
 
 	/* Add the palette */
-
-	pucTmp = (unsigned char *)pArea +
-		sizeof(sprite_area) + sizeof(sprite_header);
+	pucTmp = (UCHAR *)pArea + pArea->firstoffset + sizeof(sprite_header);
 	for (iIndex = 0; iIndex < iPaletteEntries; iIndex++) {
 		/* First color */
 		*pucTmp++ = 0;
@@ -108,11 +111,9 @@ pucCreateBlankSprite(const imagedata_type *pImg, size_t *ptSize)
 		*pucTmp++ = pImg->aucPalette[iIndex][2];
 	}
 
-	if (ptSize != NULL) {
-		*ptSize = tSize;
-	}
-	return (unsigned char *)pArea;
-} /* end of pucCreateBlankSprite */
+	*pSize = tSize;
+	return pArea;
+} /* end of pCreateBlankSprite */
 
 /*
  * iReduceColor - reduce from 24 bit to 8 bit color
@@ -141,10 +142,10 @@ iReduceColor(int iRed, int iGreen, int iBlue)
  * vDecode1bpp - decode an uncompressed 1 bit per pixel image
  */
 static void
-vDecode1bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
+vDecode1bpp(FILE *pFile, UCHAR *pucData, const imagedata_type *pImg)
 {
 	int	iX, iY, iByteWidth, iOffset, iTmp, iEighthWidth, iPadding;
-	unsigned char	ucTmp;
+	UCHAR	ucTmp;
 
 	DBG_MSG("vDecode1bpp");
 
@@ -165,18 +166,18 @@ vDecode1bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
 				return;
 			}
 			/* Reverse the bit order */
-			ucTmp  = (iTmp & BIT(0)) ? (unsigned char)BIT(7) : 0;
-			ucTmp |= (iTmp & BIT(1)) ? (unsigned char)BIT(6) : 0;
-			ucTmp |= (iTmp & BIT(2)) ? (unsigned char)BIT(5) : 0;
-			ucTmp |= (iTmp & BIT(3)) ? (unsigned char)BIT(4) : 0;
-			ucTmp |= (iTmp & BIT(4)) ? (unsigned char)BIT(3) : 0;
-			ucTmp |= (iTmp & BIT(5)) ? (unsigned char)BIT(2) : 0;
-			ucTmp |= (iTmp & BIT(6)) ? (unsigned char)BIT(1) : 0;
-			ucTmp |= (iTmp & BIT(7)) ? (unsigned char)BIT(0) : 0;
+			ucTmp  = (iTmp & BIT(0)) ? (UCHAR)BIT(7) : 0;
+			ucTmp |= (iTmp & BIT(1)) ? (UCHAR)BIT(6) : 0;
+			ucTmp |= (iTmp & BIT(2)) ? (UCHAR)BIT(5) : 0;
+			ucTmp |= (iTmp & BIT(3)) ? (UCHAR)BIT(4) : 0;
+			ucTmp |= (iTmp & BIT(4)) ? (UCHAR)BIT(3) : 0;
+			ucTmp |= (iTmp & BIT(5)) ? (UCHAR)BIT(2) : 0;
+			ucTmp |= (iTmp & BIT(6)) ? (UCHAR)BIT(1) : 0;
+			ucTmp |= (iTmp & BIT(7)) ? (UCHAR)BIT(0) : 0;
 			iOffset = iY * iByteWidth + iX;
 			*(pucData + iOffset) = ucTmp;
 		}
-		(void)iSkipBytes(pFile, iPadding);
+		(void)tSkipBytes(pFile, iPadding);
 	}
 } /* end of vDecode1bpp */
 
@@ -184,10 +185,10 @@ vDecode1bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
  * vDecode4bpp - decode an uncompressed 4 bits per pixel image
  */
 static void
-vDecode4bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
+vDecode4bpp(FILE *pFile, UCHAR *pucData, const imagedata_type *pImg)
 {
 	int	iX, iY, iByteWidth, iOffset, iTmp, iHalfWidth, iPadding;
-	unsigned char	ucTmp;
+	UCHAR	ucTmp;
 
 	DBG_MSG("vDecode4bpp");
 
@@ -213,7 +214,7 @@ vDecode4bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
 			iOffset = iY * iByteWidth + iX;
 			*(pucData + iOffset) = ucTmp;
 		}
-		(void)iSkipBytes(pFile, iPadding);
+		(void)tSkipBytes(pFile, iPadding);
 	}
 } /* end of vDecode4bpp */
 
@@ -221,7 +222,7 @@ vDecode4bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
  * vDecode8bpp - decode an uncompressed 8 bits per pixel image
  */
 static void
-vDecode8bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
+vDecode8bpp(FILE *pFile, UCHAR *pucData, const imagedata_type *pImg)
 {
 	int	iX, iY, iByteWidth, iOffset, iIndex, iPadding;
 
@@ -248,7 +249,7 @@ vDecode8bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
 				pImg->aucPalette[iIndex][1],
 				pImg->aucPalette[iIndex][2]);
 		}
-		(void)iSkipBytes(pFile, iPadding);
+		(void)tSkipBytes(pFile, iPadding);
 	}
 } /* end of vDecode8bpp */
 
@@ -256,7 +257,7 @@ vDecode8bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
  * vDecode24bpp - decode an uncompressed 24 bits per pixel image
  */
 static void
-vDecode24bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
+vDecode24bpp(FILE *pFile, UCHAR *pucData, const imagedata_type *pImg)
 {
 	int	iX, iY, iTripleWidth, iByteWidth, iOffset, iPadding;
 	int	iRed, iGreen, iBlue;
@@ -290,7 +291,7 @@ vDecode24bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
 			*(pucData + iOffset) =
 					iReduceColor(iRed, iGreen, iBlue);
 		}
-		(void)iSkipBytes(pFile, iPadding);
+		(void)tSkipBytes(pFile, iPadding);
 	}
 } /* end of vDecode24bpp */
 
@@ -298,12 +299,12 @@ vDecode24bpp(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
  * vDecodeRle4 - decode a RLE compressed 4 bits per pixel image
  */
 static void
-vDecodeRle4(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
+vDecodeRle4(FILE *pFile, UCHAR *pucData, const imagedata_type *pImg)
 {
 	int	iX, iY, iByteWidth, iOffset, iTmp, iHalfWidth;
 	int	iRun, iRunLength, iHalfRun;
 	BOOL	bEOL;
-	unsigned char	ucTmp;
+	UCHAR	ucTmp;
 
 	DBG_MSG("vDecodeRle4");
 
@@ -378,7 +379,7 @@ vDecodeRle4(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
 				}
 				/* Padding if the number of bytes is odd */
 				if (odd(iHalfRun)) {
-					(void)iSkipBytes(pFile, 1);
+					(void)tSkipBytes(pFile, 1);
 				}
 			}
 		}
@@ -390,7 +391,7 @@ vDecodeRle4(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
  * vDecodeRle8 - decode a RLE compressed 8 bits per pixel image
  */
 static void
-vDecodeRle8(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
+vDecodeRle8(FILE *pFile, UCHAR *pucData, const imagedata_type *pImg)
 {
 	int	iX, iY, iRun, iRunLength, iOffset, iIndex, iByteWidth;
 	BOOL	bEOL;
@@ -467,7 +468,7 @@ vDecodeRle8(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
 				}
 				/* Padding if the number of bytes is odd */
 				if (odd(iRunLength)) {
-					(void)iSkipBytes(pFile, 1);
+					(void)tSkipBytes(pFile, 1);
 				}
 			}
 		}
@@ -477,7 +478,7 @@ vDecodeRle8(FILE *pFile, unsigned char *pucData, const imagedata_type *pImg)
 
 #if 0 /* defined(DEBUG) */
 static void
-vCopy2File(unsigned char *pucSprite, size_t tSpriteSize)
+vCopy2File(UCHAR *pucSprite, size_t tSpriteSize)
 {
 	FILE	*pOutFile;
 	int	iIndex;
@@ -505,25 +506,27 @@ vCopy2File(unsigned char *pucSprite, size_t tSpriteSize)
 static void
 vDecodeDIB(diagram_type *pDiag, FILE *pFile, const imagedata_type *pImg)
 {
-	unsigned char	*pucSprite, *pucPalette, *pucData;
+	sprite_areainfo	*pSprite;
+	UCHAR	*pucPalette, *pucData;
 	size_t	tSpriteSize;
 	int	iHeaderSize;
 
 	/* Skip the bitmap info header */
 	iHeaderSize = (int)ulNextLong(pFile);
-	(void)iSkipBytes(pFile, iHeaderSize - 4);
+	(void)tSkipBytes(pFile, iHeaderSize - 4);
 	/* Skip the colortable */
-	if (pImg->iBitsPerComponent <= 8) {
-		(void)iSkipBytes(pFile,
+	if (pImg->uiBitsPerComponent <= 8) {
+		(void)tSkipBytes(pFile,
 			pImg->iColorsUsed * ((iHeaderSize > 12) ? 4 : 3));
 	}
 
-	/* Create an empty sprite */
-	pucSprite = pucCreateBlankSprite(pImg, &tSpriteSize);
-	pucPalette = pucSprite + sizeof(sprite_area) + sizeof(sprite_header);
+	/* Create an blank sprite */
+	pSprite = pCreateBlankSprite(pImg, &tSpriteSize);
+	pucPalette = (UCHAR *)pSprite +
+			pSprite->firstoffset + sizeof(sprite_header);
 
 	/* Add the pixel information */
-	switch (pImg->iBitsPerComponent) {
+	switch (pImg->uiBitsPerComponent) {
 	case  1:
 		fail(pImg->eCompression != compression_none);
 		pucData = pucPalette + 2 * 8;
@@ -542,7 +545,7 @@ vDecodeDIB(diagram_type *pDiag, FILE *pFile, const imagedata_type *pImg)
 	case  8:
 		fail(pImg->eCompression != compression_none &&
 				pImg->eCompression != compression_rle8);
-		pucData = pucPalette;
+		pucData = pucPalette + 0 * 8;
 		if (pImg->eCompression == compression_rle8) {
 			vDecodeRle8(pFile, pucData, pImg);
 		} else {
@@ -551,25 +554,25 @@ vDecodeDIB(diagram_type *pDiag, FILE *pFile, const imagedata_type *pImg)
 		break;
 	case 24:
 		fail(pImg->eCompression != compression_none);
-		pucData = pucPalette;
+		pucData = pucPalette + 0 * 8;
 		vDecode24bpp(pFile, pucData, pImg);
 		break;
 	default:
-		DBG_DEC(pImg->iBitsPerComponent);
+		DBG_DEC(pImg->uiBitsPerComponent);
 		break;
 	}
 
 #if 0 /* defined(DEBUG) */
-	vCopy2File(pucSprite, tSpriteSize);
+	vCopy2File((UCHAR *)pSprite, tSpriteSize);
 #endif /* DEBUG */
 
 	/* Add the sprite to the Draw file */
 	vImage2Diagram(pDiag, pImg,
-		pucSprite + sizeof(sprite_area),
-		tSpriteSize - sizeof(sprite_area));
+		(UCHAR *)pSprite + pSprite->firstoffset,
+		tSpriteSize - pSprite->firstoffset);
 
 	/* Clean up before you leave */
-	pucSprite = xfree(pucSprite);
+	pSprite = xfree(pSprite);
 } /* end of vDecodeDIB */
 
 /*
@@ -581,10 +584,10 @@ vDecodeDIB(diagram_type *pDiag, FILE *pFile, const imagedata_type *pImg)
  */
 BOOL
 bTranslateDIB(diagram_type *pDiag, FILE *pFile,
-		long lFileOffset, const imagedata_type *pImg)
+	ULONG ulFileOffset, const imagedata_type *pImg)
 {
-	/* seek to start position of DIB data */
-	if (!bSetDataOffset(pFile, lFileOffset)) {
+	/* Seek to start position of DIB data */
+	if (!bSetDataOffset(pFile, ulFileOffset)) {
 		return FALSE;
 	}
 
