@@ -1,6 +1,6 @@
 /*
  * imgexam.c
- * Copyright (C) 2000,2001 A.J. van Os; Released under GPL
+ * Copyright (C) 2000-2004 A.J. van Os; Released under GNU GPL
  *
  * Description:
  * Functions to examine image headers
@@ -28,6 +28,14 @@
 #define PNG_CB_COLOR		0x02
 #define PNG_CB_ALPHA		0x04
 
+/* Instance signature */
+#define MSOBI_WMF	0x0216
+#define MSOBI_EMF	0x03d4
+#define MSOBI_PICT	0x0542
+#define MSOBI_PNG	0x06e0
+#define MSOBI_JPEG	0x046a
+#define MSOBI_DIB	0x07a8
+
 /* The following enum is stolen from the IJG JPEG library */
 typedef enum {		/* JPEG marker codes			*/
 	M_SOF0	= 0xc0,	/* baseline DCT				*/
@@ -109,13 +117,14 @@ bFillPaletteDIB(FILE *pFile, imagedata_type *pImg, BOOL bNewFormat)
 	fail(pFile == NULL);
 	fail(pImg == NULL);
 
-	if (pImg->iBitsPerComponent > 8) {
-		/* No palette, images uses more than 256 colors */
+	if (pImg->uiBitsPerComponent > 8) {
+		/* No palette, image uses more than 256 colors */
 		return TRUE;
 	}
 
 	if (pImg->iColorsUsed <= 0) {
-		pImg->iColorsUsed = 1 << pImg->iBitsPerComponent;
+		/* Not specified, so compute the number of colors used */
+		pImg->iColorsUsed = 1 << pImg->uiBitsPerComponent;
 	}
 
 	fail(pImg->iColorsUsed > 256);
@@ -161,7 +170,7 @@ bExamineDIB(FILE *pFile, imagedata_type *pImg)
 		pImg->iWidth = (int)usNextWord(pFile);
 		pImg->iHeight = (int)usNextWord(pFile);
 		iPlanes = (int)usNextWord(pFile);
-		pImg->iBitsPerComponent = (int)usNextWord(pFile);
+		pImg->uiBitsPerComponent = (UINT)usNextWord(pFile);
 		iCompression = BI_RGB;
 		pImg->iColorsUsed = 0;
 		break;
@@ -170,11 +179,11 @@ bExamineDIB(FILE *pFile, imagedata_type *pImg)
 		pImg->iWidth = (int)ulNextLong(pFile);
 		pImg->iHeight = (int)ulNextLong(pFile);
 		iPlanes = (int)usNextWord(pFile);
-		pImg->iBitsPerComponent = (int)usNextWord(pFile);
+		pImg->uiBitsPerComponent = (UINT)usNextWord(pFile);
 		iCompression = (int)ulNextLong(pFile);
-		(void)iSkipBytes(pFile, 12);
+		(void)tSkipBytes(pFile, 12);
 		pImg->iColorsUsed = (int)ulNextLong(pFile);
-		(void)iSkipBytes(pFile, tHeaderSize - 36);
+		(void)tSkipBytes(pFile, tHeaderSize - 36);
 		break;
 	default:
 		DBG_DEC(tHeaderSize);
@@ -182,7 +191,7 @@ bExamineDIB(FILE *pFile, imagedata_type *pImg)
 	}
 	DBG_DEC(pImg->iWidth);
 	DBG_DEC(pImg->iHeight);
-	DBG_DEC(pImg->iBitsPerComponent);
+	DBG_DEC(pImg->uiBitsPerComponent);
 	DBG_DEC(iCompression);
 	DBG_DEC(pImg->iColorsUsed);
 
@@ -196,19 +205,19 @@ bExamineDIB(FILE *pFile, imagedata_type *pImg)
 		DBG_DEC(pImg->iHeight);
 		return FALSE;
 	}
-	if (pImg->iBitsPerComponent != 1 && pImg->iBitsPerComponent != 4 &&
-	    pImg->iBitsPerComponent != 8 && pImg->iBitsPerComponent != 24) {
-		DBG_DEC(pImg->iBitsPerComponent);
+	if (pImg->uiBitsPerComponent != 1 && pImg->uiBitsPerComponent != 4 &&
+	    pImg->uiBitsPerComponent != 8 && pImg->uiBitsPerComponent != 24) {
+		DBG_DEC(pImg->uiBitsPerComponent);
 		return FALSE;
 	}
 	if (iCompression != BI_RGB &&
-	    (pImg->iBitsPerComponent == 1 || pImg->iBitsPerComponent == 24)) {
+	    (pImg->uiBitsPerComponent == 1 || pImg->uiBitsPerComponent == 24)) {
 		return FALSE;
 	}
-	if (iCompression == BI_RLE8 && pImg->iBitsPerComponent == 4) {
+	if (iCompression == BI_RLE8 && pImg->uiBitsPerComponent == 4) {
 		return FALSE;
 	}
-	if (iCompression == BI_RLE4 && pImg->iBitsPerComponent == 8) {
+	if (iCompression == BI_RLE4 && pImg->uiBitsPerComponent == 8) {
 		return FALSE;
 	}
 
@@ -229,10 +238,10 @@ bExamineDIB(FILE *pFile, imagedata_type *pImg)
 
 	pImg->bColorImage = bFillPaletteDIB(pFile, pImg, tHeaderSize > 12);
 
-	if (pImg->iBitsPerComponent <= 8) {
+	if (pImg->uiBitsPerComponent <= 8) {
 		pImg->iComponents = 1;
 	} else {
-		pImg->iComponents = pImg->iBitsPerComponent / 8;
+		pImg->iComponents = (int)(pImg->uiBitsPerComponent / 8);
 	}
 
 	return TRUE;
@@ -300,7 +309,7 @@ bExamineJPEG(FILE *pFile, imagedata_type *pImg)
 		case M_SOF0:
 		case M_SOF1:
 			tLength = (size_t)usNextWordBE(pFile);
-			pImg->iBitsPerComponent = iNextByte(pFile);
+			pImg->uiBitsPerComponent = (UINT)iNextByte(pFile);
 			pImg->iHeight = (int)usNextWordBE(pFile);
 			pImg->iWidth = (int)usNextWordBE(pFile);
 			pImg->iComponents = iNextByte(pFile);
@@ -314,7 +323,7 @@ bExamineJPEG(FILE *pFile, imagedata_type *pImg)
 		 */
 			tLength = (size_t)usNextWordBE(pFile);
 			if (tLength < 12) {
-				(void)iSkipBytes(pFile, tLength - 2);
+				(void)tSkipBytes(pFile, tLength - 2);
 			} else {
 				for (iIndex = 0; iIndex < 5; iIndex++) {
 					appstring[iIndex] =
@@ -324,7 +333,7 @@ bExamineJPEG(FILE *pFile, imagedata_type *pImg)
 				if (STREQ(appstring, "Adobe")) {
 					pImg->bAdobe = TRUE;
 				}
-				(void)iSkipBytes(pFile, tLength - 7);
+				(void)tSkipBytes(pFile, tLength - 7);
 			}
 			break;
 		case M_SOI:		/* ignore markers without parameters */
@@ -341,14 +350,14 @@ bExamineJPEG(FILE *pFile, imagedata_type *pImg)
 			break;
 		default:		/* skip variable length markers */
 			tLength = (size_t)usNextWordBE(pFile);
-			(void)iSkipBytes(pFile, tLength - 2);
+			(void)tSkipBytes(pFile, tLength - 2);
 			break;
 		}
 	}
 
 	DBG_DEC(pImg->iWidth);
 	DBG_DEC(pImg->iHeight);
-	DBG_DEC(pImg->iBitsPerComponent);
+	DBG_DEC(pImg->uiBitsPerComponent);
 	DBG_DEC(pImg->iComponents);
 
 	/* Do some sanity checks with the parameters */
@@ -366,8 +375,8 @@ bExamineJPEG(FILE *pFile, imagedata_type *pImg)
 		DBG_MSG("Warning: SOF marker has incorrect length - ignored");
 	}
 
-	if (pImg->iBitsPerComponent != 8) {
-		DBG_DEC(pImg->iBitsPerComponent);
+	if (pImg->uiBitsPerComponent != 8) {
+		DBG_DEC(pImg->uiBitsPerComponent);
 		DBG_MSG("Not supported in PostScript level 2");
 		return FALSE;
 	}
@@ -399,7 +408,7 @@ bFillPalettePNG(FILE *pFile, imagedata_type *pImg, size_t tLength)
 	fail(pFile == NULL);
 	fail(pImg == NULL);
 
-	if (pImg->iBitsPerComponent > 8) {
+	if (pImg->uiBitsPerComponent > 8) {
 		/* No palette, image uses more than 256 colors */
 		return TRUE;
 	}
@@ -415,9 +424,9 @@ bFillPalettePNG(FILE *pFile, imagedata_type *pImg, size_t tLength)
 		return FALSE;
 	}
 
-	iEntries = tLength / 3;
+	iEntries = (int)(tLength / 3);
 	DBG_DEC(iEntries);
-	pImg->iColorsUsed = 1 << pImg->iBitsPerComponent;
+	pImg->iColorsUsed = 1 << pImg->uiBitsPerComponent;
 	DBG_DEC(pImg->iColorsUsed);
 
 	if (iEntries > 256) {
@@ -491,13 +500,13 @@ bExaminePNG(FILE *pFile, imagedata_type *pImg)
 			}
 			pImg->iWidth = (int)ulNextLongBE(pFile);
 			pImg->iHeight = (int)ulNextLongBE(pFile);
-			pImg->iBitsPerComponent = iNextByte(pFile);
+			pImg->uiBitsPerComponent = (UINT)iNextByte(pFile);
 			iTmp = iNextByte(pFile);
 			NO_DBG_HEX(iTmp);
 			pImg->bColorImage = (iTmp & PNG_CB_COLOR) != 0;
 			bHasPalette = (iTmp & PNG_CB_PALETTE) != 0;
 			bHasAlpha = (iTmp & PNG_CB_ALPHA) != 0;
-			if (bHasPalette && pImg->iBitsPerComponent > 8) {
+			if (bHasPalette && pImg->uiBitsPerComponent > 8) {
 				/* This should not happen */
 				return FALSE;
 			}
@@ -522,7 +531,7 @@ bExaminePNG(FILE *pFile, imagedata_type *pImg)
 				return FALSE;
 			}
 			pImg->iColorsUsed = 0;
-			(void)iSkipBytes(pFile, tLength - 13 + 4);
+			(void)tSkipBytes(pFile, tLength - 13 + 4);
 			break;
 		case PNG_CN_PLTE:
 			if (!bHasPalette) {
@@ -531,17 +540,17 @@ bExaminePNG(FILE *pFile, imagedata_type *pImg)
 			if (!bFillPalettePNG(pFile, pImg, tLength)) {
 				return FALSE;
 			}
-			(void)iSkipBytes(pFile, 4);
+			(void)tSkipBytes(pFile, 4);
 			break;
 		default:
-			(void)iSkipBytes(pFile, tLength + 4);
+			(void)tSkipBytes(pFile, tLength + 4);
 			break;
 		}
 	}
 
 	DBG_DEC(pImg->iWidth);
 	DBG_DEC(pImg->iHeight);
-	DBG_DEC(pImg->iBitsPerComponent);
+	DBG_DEC(pImg->uiBitsPerComponent);
 	DBG_DEC(pImg->iColorsUsed);
 	DBG_DEC(pImg->iComponents);
 
@@ -550,10 +559,10 @@ bExaminePNG(FILE *pFile, imagedata_type *pImg)
 		return FALSE;
 	}
 
-	if (pImg->iBitsPerComponent != 1 && pImg->iBitsPerComponent != 2 &&
-	    pImg->iBitsPerComponent != 4 && pImg->iBitsPerComponent != 8 &&
-	    pImg->iBitsPerComponent != 16) {
-		DBG_DEC(pImg->iBitsPerComponent);
+	if (pImg->uiBitsPerComponent != 1 && pImg->uiBitsPerComponent != 2 &&
+	    pImg->uiBitsPerComponent != 4 && pImg->uiBitsPerComponent != 8 &&
+	    pImg->uiBitsPerComponent != 16) {
+		DBG_DEC(pImg->uiBitsPerComponent);
 		return  FALSE;
 	}
 
@@ -563,20 +572,20 @@ bExaminePNG(FILE *pFile, imagedata_type *pImg)
 		return FALSE;
 	}
 
-	if (pImg->iBitsPerComponent > 8) {
+	if (pImg->uiBitsPerComponent > 8) {
 		/* Not supported */
-		DBG_DEC(pImg->iBitsPerComponent);
+		DBG_DEC(pImg->uiBitsPerComponent);
 		return FALSE;
 	}
 
 	if (pImg->iColorsUsed == 0 &&
 	    pImg->iComponents == 1 &&
-	    pImg->iBitsPerComponent <= 4) {
+	    pImg->uiBitsPerComponent <= 4) {
 		/*
-		 * No palette is supplied, but Postscript needs one in these
+		 * No palette is supplied, but PostScript needs one in these
 		 * cases, so we add a default palette here
 		 */
-		pImg->iColorsUsed = 1 << pImg->iBitsPerComponent;
+		pImg->iColorsUsed = 1 << pImg->uiBitsPerComponent;
 		iIncrement = 0xff / (pImg->iColorsUsed - 1);
 		for (iIndex = 0, iColor = 0x00;
 		     iIndex < pImg->iColorsUsed;
@@ -675,197 +684,207 @@ vImage2Papersize(imagedata_type *pImg)
 #endif /* !__riscos */
 
 /*
- * iFind6Image - skip until the image is found
+ * tFind6Image - skip until the image is found
  *
  * Find the image in Word 6/7 files
  *
  * returns the new position when a image is found, otherwise -1
  */
-static int
-iFind6Image(FILE *pFile, int iPos, int iLen, imagetype_enum *peImageType)
+static size_t
+tFind6Image(FILE *pFile, size_t tPosition, size_t tLength,
+	imagetype_enum *peImageType)
 {
 	ULONG	ulMarker;
-	int	iElementLen, iToSkip;
+	size_t	tRecordLength, tToSkip;
 	USHORT	usMarker;
 
 	fail(pFile == NULL);
-	fail(iPos < 0);
-	fail(iLen < 0);
 	fail(peImageType == NULL);
 
 	*peImageType = imagetype_is_unknown;
-	if (iPos + 18 >= iLen) {
-		return -1;
+	if (tPosition + 18 >= tLength) {
+		return (size_t)-1;
 	}
 
 	ulMarker = ulNextLong(pFile);
 	if (ulMarker != 0x00090001) {
 		DBG_HEX(ulMarker);
-		return -1;
+		return (size_t)-1;
 	}
 	usMarker = usNextWord(pFile);
 	if (usMarker != 0x0300) {
 		DBG_HEX(usMarker);
-		return -1;
+		return (size_t)-1;
 	}
-	(void)iSkipBytes(pFile, 10);
+	(void)tSkipBytes(pFile, 10);
 	usMarker = usNextWord(pFile);
 	if (usMarker != 0x0000) {
 		DBG_HEX(usMarker);
-		return -1;
+		return (size_t)-1;
 	}
-	iPos += 18;
+	tPosition += 18;
 
-	while (iPos + 6 < iLen) {
-		iElementLen = (int)ulNextLong(pFile);
+	while (tPosition + 6 <= tLength) {
+		tRecordLength = (size_t)ulNextLong(pFile);
 		usMarker = usNextWord(pFile);
-		iPos += 6;
-		DBG_DEC(iElementLen);
-		DBG_HEX(usMarker);
-		if (iElementLen == 3) {
-#if 0
-			DBG_MSG("DIB");
-			*peImageType = imagetype_is_dib;
-			return iPos;
-#else
-			DBG_DEC(iPos);
-			return -1;
-#endif
-		}
-
+		tPosition += 6;
+		NO_DBG_DEC(tRecordLength);
+		NO_DBG_HEX(usMarker);
 		switch (usMarker) {
+		case 0x0000:
+			DBG_HEX(ulGetDataOffset(pFile));
+			return (size_t)-1;
 		case 0x0b41:
 			DBG_MSG("DIB");
 			*peImageType = imagetype_is_dib;
-			iPos += iSkipBytes(pFile, 20);
-			return iPos;
+			tPosition += tSkipBytes(pFile, 20);
+			return tPosition;
 		case 0x0f43:
 			DBG_MSG("DIB");
 			*peImageType = imagetype_is_dib;
-			iPos += iSkipBytes(pFile, 22);
-			return iPos;
+			tPosition += tSkipBytes(pFile, 22);
+			return tPosition;
 		default:
-			if (iElementLen > INT_MAX / 2 + 3) {
+			if (tRecordLength < 3) {
+				break;
+			}
+			if (tRecordLength > SIZE_T_MAX / 2) {
 				/*
-				 * This value is so big the number of bytes
-				 * to skip can not be computed
+				 * No need to compute the number of bytes
+				 * to skip
 				 */
-				DBG_DEC(iElementLen);
-				return -1;
+				DBG_DEC(tRecordLength);
+				DBG_HEX(tRecordLength);
+				DBG_FIXME();
+				return (size_t)-1;
 			}
-			iToSkip = (iElementLen - 3) * 2;
-			if (iToSkip <= 0 || iToSkip > iLen - iPos) {
+			tToSkip = tRecordLength * 2 - 6;
+			if (tToSkip > tLength - tPosition) {
 				/* You can't skip this number of bytes */
-				DBG_DEC(iToSkip);
-				DBG_DEC(iLen - iPos);
-				return -1;
+				DBG_DEC(tToSkip);
+				DBG_DEC(tLength - tPosition);
+				return (size_t)-1;
 			}
-			iPos += iSkipBytes(pFile, (size_t)iToSkip);
+			tPosition += tSkipBytes(pFile, tToSkip);
 			break;
 		}
 	}
 
-	return -1;
-} /* end of iFind6Image */
+	return (size_t)-1;
+} /* end of tFind6Image */
 
 /*
- * iFind8Image - skip until the image is found
+ * tFind8Image - skip until the image is found
  *
- * Find the image in Word 8/97 files
+ * Find the image in Word 8/9/10 files
  *
  * returns the new position when a image is found, otherwise -1
  */
-static int
-iFind8Image(FILE *pFile, int iPos, int iLen, imagetype_enum *peImageType)
+static size_t
+tFind8Image(FILE *pFile, size_t tPosition, size_t tLength,
+	imagetype_enum *peImageType)
 {
-	int	iElementTag, iElementLen;
-	USHORT	usID;
+	size_t	tRecordLength, tNameLen;
+	USHORT	usRecordVersion, usRecordType, usRecordInstance;
+	USHORT	usTmp;
 
 	fail(pFile == NULL);
-	fail(iPos < 0);
-	fail(iLen < 0);
 	fail(peImageType == NULL);
 
 	*peImageType = imagetype_is_unknown;
-	while (iPos + 8 < iLen) {
-		usID = usNextWord(pFile) >> 4;
-		iElementTag = (int)usNextWord(pFile);
-		iElementLen = (int)ulNextLong(pFile);
-		iPos += 8;
-		NO_DBG_HEX(usID);
-		NO_DBG_HEX(iElementTag);
-		NO_DBG_DEC(iElementLen);
-		switch (iElementTag) {
-		case 0xf001:
-		case 0xf002:
-		case 0xf003:
-		case 0xf004:
-		case 0xf005:
+	while (tPosition + 8 <= tLength) {
+		usTmp = usNextWord(pFile);
+		usRecordVersion = usTmp & 0x000f;
+		usRecordInstance = usTmp >> 4;
+		usRecordType = usNextWord(pFile);
+		tRecordLength = (size_t)ulNextLong(pFile);
+		tPosition += 8;
+		NO_DBG_HEX(usRecordVersion);
+		NO_DBG_HEX(usRecordInstance);
+		NO_DBG_HEX(usRecordType);
+		NO_DBG_DEC(tRecordLength);
+		switch (usRecordType) {
+		case 0xf000: case 0xf001: case 0xf002: case 0xf003:
+		case 0xf004: case 0xf005:
 			break;
 		case 0xf007:
-			iPos += iSkipBytes(pFile, 36);
+			tPosition += tSkipBytes(pFile, 33);
+			tNameLen = (size_t)iNextByte(pFile);
+			tPosition++;
+			DBG_DEC_C(tNameLen != 0, tNameLen);
+			tPosition += tSkipBytes(pFile, 2 + tNameLen * 2);
 			break;
 		case 0xf008:
-			iPos += iSkipBytes(pFile, 8);
+			tPosition += tSkipBytes(pFile, 8);
 			break;
 		case 0xf009:
-			iPos += iSkipBytes(pFile, 16);
+			tPosition += tSkipBytes(pFile, 16);
 			break;
-		case 0xf000:
-		case 0xf006:
-		case 0xf00a:
-		case 0xf00b:
-		case 0xf00d:
-		case 0xf00e:
-		case 0xf00f:
-		case 0xf010:
-		case 0xf011:
-			if (iElementLen < 0) {
-				DBG_DEC(iElementLen);
-				return -1;
-			}
-			iPos += iSkipBytes(pFile, (size_t)iElementLen);
+		case 0xf006: case 0xf00a: case 0xf00b: case 0xf00d:
+		case 0xf00e: case 0xf00f: case 0xf010: case 0xf011:
+		case 0xf122:
+			tPosition += tSkipBytes(pFile, tRecordLength);
 			break;
 		case 0xf01a:
 			DBG_MSG("EMF");
 			*peImageType = imagetype_is_emf;
-			iPos += iSkipBytes(pFile, usID == 0x3d4 ? 50 : 66);
-			return iPos;
+			tPosition += tSkipBytes(pFile, 50);
+			if ((usRecordInstance ^ MSOBI_EMF) == 1) {
+				tPosition += tSkipBytes(pFile, 16);
+			}
+			return tPosition;
 		case 0xf01b:
 			DBG_MSG("WMF");
 			*peImageType = imagetype_is_wmf;
-			iPos += iSkipBytes(pFile, usID == 0x216 ? 50 : 66);
-			return iPos;
+			tPosition += tSkipBytes(pFile, 50);
+			if ((usRecordInstance ^ MSOBI_WMF) == 1) {
+				tPosition += tSkipBytes(pFile, 16);
+			}
+			return tPosition;
 		case 0xf01c:
 			DBG_MSG("PICT");
 			*peImageType = imagetype_is_pict;
-			iPos += iSkipBytes(pFile, usID == 0x542 ? 17 : 33) ;
-			return iPos;
+			tPosition += tSkipBytes(pFile, 50);
+			if ((usRecordInstance ^ MSOBI_PICT) == 1) {
+				tPosition += tSkipBytes(pFile, 16);
+			}
+			return tPosition;
 		case 0xf01d:
 			DBG_MSG("JPEG");
 			*peImageType = imagetype_is_jpeg;
-			iPos += iSkipBytes(pFile, usID == 0x46a ? 17 : 33);
-			return iPos;
+			tPosition += tSkipBytes(pFile, 17);
+			if ((usRecordInstance ^ MSOBI_JPEG) == 1) {
+				tPosition += tSkipBytes(pFile, 16);
+			}
+			return tPosition;
 		case 0xf01e:
 			DBG_MSG("PNG");
 			*peImageType = imagetype_is_png;
-			iPos += iSkipBytes(pFile, usID == 0x6e0 ? 17 : 33);
-			return iPos;
+			tPosition += tSkipBytes(pFile, 17);
+			if ((usRecordInstance ^ MSOBI_PNG) == 1) {
+				tPosition += tSkipBytes(pFile, 16);
+			}
+			return tPosition;
 		case 0xf01f:
 			DBG_MSG("DIB");
 			/* DIB is a BMP minus its 14 byte header */
 			*peImageType = imagetype_is_dib;
-			iPos += iSkipBytes(pFile, usID == 0x7a8 ? 17 : 33);
-			return iPos;
+			tPosition += tSkipBytes(pFile, 17);
+			if ((usRecordInstance ^ MSOBI_DIB) == 1) {
+				tPosition += tSkipBytes(pFile, 16);
+			}
+			return tPosition;
 		case 0xf00c:
 		default:
-			DBG_HEX(iElementTag);
-			return -1;
+			DBG_HEX(usRecordType);
+			DBG_DEC_C(tRecordLength % 4 != 0, tRecordLength);
+			DBG_FIXME();
+			return (size_t)-1;
 		}
 	}
 
-	return -1;
-} /* end of iFind8Image */
+	return (size_t)-1;
+} /* end of tFind8Image */
 
 /*
  * eExamineImage - Examine the image
@@ -876,8 +895,7 @@ image_info_enum
 eExamineImage(FILE *pFile, ULONG ulFileOffsetImage, imagedata_type *pImg)
 {
 	long	lTmp;
-	size_t	tWordHeaderLen;
-	int	iLen, iPos;
+	size_t	tWordHeaderLen, tLength, tPos;
 	int	iType, iHorSize, iVerSize;
 	USHORT	usHorScalingFactor, usVerScalingFactor;
 
@@ -890,23 +908,26 @@ eExamineImage(FILE *pFile, ULONG ulFileOffsetImage, imagedata_type *pImg)
 		return image_no_information;
 	}
 
-	iLen = (int)ulNextLong(pFile);
-	DBG_DEC(iLen);
-	if (iLen < 58) {
+	tLength = (size_t)ulNextLong(pFile);
+	DBG_DEC(tLength);
+	if (tLength < 46) {
 		/* Smaller than the smallest known header */
+		DBG_FIXME();
 		return image_no_information;
 	}
 	tWordHeaderLen = (size_t)usNextWord(pFile);
 	DBG_DEC(tWordHeaderLen);
-	fail(tWordHeaderLen != 58 && tWordHeaderLen != 68);
+	fail(tWordHeaderLen != 46 &&
+		tWordHeaderLen != 58 &&
+		tWordHeaderLen != 68);
 
-	if (iLen < (int)tWordHeaderLen) {
+	if (tLength < tWordHeaderLen) {
 		/* Smaller than the current header */
 		return image_no_information;
 	}
 	iType = (int)usNextWord(pFile);
 	DBG_DEC(iType);
-	(void)iSkipBytes(pFile, 28 - 8);
+	(void)tSkipBytes(pFile, 28 - 8);
 
 	lTmp = lTwips2MilliPoints(usNextWord(pFile));
 	iHorSize = (int)(lTmp / 1000);
@@ -941,47 +962,47 @@ eExamineImage(FILE *pFile, ULONG ulFileOffsetImage, imagedata_type *pImg)
 	}
 
 	/* Skip the rest of the header */
-	(void)iSkipBytes(pFile, tWordHeaderLen - 36);
-	iPos = (int)tWordHeaderLen;
+	(void)tSkipBytes(pFile, tWordHeaderLen - 36);
+	tPos = tWordHeaderLen;
 
 	(void)memset(pImg, 0, sizeof(*pImg));
 
 	switch (iType) {
 	case   7:
 	case   8:
-		iPos = iFind6Image(pFile, iPos, iLen, &pImg->eImageType);
-		if (iPos < 0) {
+		tPos = tFind6Image(pFile, tPos, tLength, &pImg->eImageType);
+		if (tPos == (size_t)-1) {
 			/* No image found */
 			return image_no_information;
 		}
-		DBG_HEX(iPos);
+		DBG_HEX(tPos);
 		break;
 	case  94:	/* Word 6/7, no image just a pathname */
 		pImg->eImageType = imagetype_is_external;
-		DBG_HEX(ulFileOffsetImage + iPos);
+		DBG_HEX(ulFileOffsetImage + tPos);
 		break;
 	case 100:
-		iPos = iFind8Image(pFile, iPos, iLen, &pImg->eImageType);
-		if (iPos < 0) {
+		tPos = tFind8Image(pFile, tPos, tLength, &pImg->eImageType);
+		if (tPos == (size_t)-1) {
 			/* No image found */
 			return image_no_information;
 		}
-		DBG_HEX(iPos);
+		DBG_HEX(tPos);
 		break;
-	case 102:	/* Word 8/97, no image just a pathname or URL */
+	case 102:	/* Word 8/9/10, no image just a pathname or URL */
 		pImg->eImageType = imagetype_is_external;
-		DBG_HEX(ulFileOffsetImage + iPos);
+		DBG_HEX(ulFileOffsetImage + tPos);
 		break;
 	default:
 		DBG_DEC(iType);
-		DBG_HEX(ulFileOffsetImage + iPos);
+		DBG_HEX(ulFileOffsetImage + tPos);
 		DBG_FIXME();
 		return image_no_information;
 	}
 
 	/* Minimal information is now available */
-	pImg->iLength = iLen;
-	pImg->iPosition = iPos;
+	pImg->tLength = tLength;
+	pImg->tPosition = tPos;
 	pImg->iHorSizeScaled =
 		(int)(((long)iHorSize * (long)usHorScalingFactor + 500) / 1000);
 	pImg->iVerSizeScaled =
