@@ -1,6 +1,6 @@
 /*
  * options.c
- * Copyright (C) 1998-2003 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2004 A.J. van Os; Released under GNU GPL
  *
  * Description:
  * Read and write the options
@@ -10,7 +10,8 @@
 #include <stdlib.h>
 #include <string.h>
 #if defined(__riscos)
-#include "wimpt.h"
+#include "DeskLib:Error.h"
+#include "DeskLib:Wimp.h"
 #else
 #include <stdlib.h>
 #if defined(__dos) || defined(N_PLAT_NLM)
@@ -28,6 +29,8 @@ extern int getopt(int, char **, const char *);
 #define SHOW_IMAGES		"set show_images=%d"
 #define HIDE_HIDDEN_TEXT	"set hide_hidden_text=%d"
 #define SCALE_FACTOR_START	"set scale_factor_start=%d"
+#else
+#define LEAFNAME_SIZE		(32+1)
 #endif /* __riscos */
 
 /* Current values for options */
@@ -68,9 +71,10 @@ static const options_type	tOptionsDefault = {
 #else
 	conversion_text,
 #endif /* __riscos */
+	TRUE,
 	TRUE,
 	FALSE,
-	encoding_iso_8859_1,
+	encoding_latin_1,
 	INT_MAX,
 	INT_MAX,
 	level_default,
@@ -81,6 +85,189 @@ static const options_type	tOptionsDefault = {
 };
 
 
+#if !defined(__riscos)
+/*
+ * bCorrectPapersize - see if the papersize is correct
+ *
+ * TRUE if the papersize is correct, otherwise FALSE
+ */
+static BOOL
+bCorrectPapersize(const char *szName, conversion_type eConversionType)
+{
+	const papersize_type	*pPaperSize;
+
+	for (pPaperSize = atPaperSizes;
+	     pPaperSize->szName[0] != '\0';
+	     pPaperSize++) {
+		if (!STRCEQ(pPaperSize->szName,  szName)) {
+			continue;
+		}
+		DBG_DEC(pPaperSize->usWidth);
+		DBG_DEC(pPaperSize->usHeight);
+		tOptionsCurr.eConversionType = eConversionType;
+		tOptionsCurr.iPageHeight = (int)pPaperSize->usHeight;
+		tOptionsCurr.iPageWidth = (int)pPaperSize->usWidth;
+		return TRUE;
+	}
+	return FALSE;
+} /* end of bCorrectPapersize */
+
+/*
+ * szCreateSuffix - create a suffix for the file
+ *
+ * Returns the suffix
+ */
+static const char *
+szCreateSuffix(const char *szLeafname)
+{
+	const char	*pcDot;
+
+	pcDot = strrchr(szLeafname, '.');
+	if (pcDot != NULL && STRCEQ(pcDot, ".txt")) {
+		/* There is already a .txt suffix, no need for another one */
+		return "";
+	}
+	return ".txt";
+} /* end of szCreateSuffix */
+
+/*
+ * eMappingFile2Encoding - convert the mapping file to an encoding
+ */
+static encoding_type
+eMappingFile2Encoding(const char *szLeafname)
+{
+	char	szMappingFile[LEAFNAME_SIZE+4];
+
+	fail(szLeafname == NULL);
+
+	if (strlen(szLeafname) + 4 >= sizeof(szMappingFile)) {
+		DBG_MSG(szLeafname);
+		return encoding_latin_1;
+	}
+
+	sprintf(szMappingFile, "%s%s", szLeafname, szCreateSuffix(szLeafname));
+
+	DBG_MSG(szMappingFile);
+
+	if (STRCEQ(szMappingFile, MAPPING_FILE_UTF_8)) {
+		return encoding_utf_8;
+	}
+	if (STRCEQ(szMappingFile, MAPPING_FILE_CP852) ||
+	    STRCEQ(szMappingFile, MAPPING_FILE_CP1250) ||
+	    STRCEQ(szMappingFile, MAPPING_FILE_8859_2)) {
+		return encoding_latin_2;
+	}
+	if (STRCEQ(szMappingFile, MAPPING_FILE_KOI8_R) ||
+	    STRCEQ(szMappingFile, MAPPING_FILE_KOI8_U) ||
+	    STRCEQ(szMappingFile, MAPPING_FILE_CP866) ||
+	    STRCEQ(szMappingFile, MAPPING_FILE_CP1251) ||
+	    STRCEQ(szMappingFile, MAPPING_FILE_8859_5)) {
+		return encoding_cyrillic;
+	}
+	return encoding_latin_1;
+} /* end of eMappingFile2Encoding */
+#endif /* !__riscos */
+
+/*
+ * pOpenCharacterMappingFile - open the mapping file
+ *
+ * Returns the file pointer or NULL
+ */
+static FILE *
+pOpenCharacterMappingFile(const char *szLeafname)
+{
+#if !defined(__riscos)
+	FILE	*pFile;
+	const char	*szHome, *szAntiword, *szSuffix;
+	size_t	tFilenameLen;
+	char	szMappingFile[PATH_MAX+1];
+#endif /* !__riscos */
+
+	if (szLeafname == NULL || szLeafname[0] == '\0') {
+		return NULL;
+	}
+
+	DBG_MSG(szLeafname);
+
+#if defined(__riscos)
+	return fopen(szLeafname, "r");
+#else
+	/* Set the suffix */
+	szSuffix = szCreateSuffix(szLeafname);
+
+	/* Set length */
+	tFilenameLen = strlen(szLeafname) + strlen(szSuffix);
+
+	/* Try the environment version of the mapping file */
+	szAntiword = szGetAntiwordDirectory();
+	if (szAntiword != NULL && szAntiword[0] != '\0') {
+	    if (strlen(szAntiword) + tFilenameLen <
+		sizeof(szMappingFile) -
+		sizeof(FILE_SEPARATOR)) {
+			sprintf(szMappingFile,
+				"%s" FILE_SEPARATOR "%s%s",
+				szAntiword, szLeafname, szSuffix);
+			DBG_MSG(szMappingFile);
+			pFile = fopen(szMappingFile, "r");
+			if (pFile != NULL) {
+				return pFile;
+			}
+		} else {
+			werr(0, "Environment mappingfilename ignored");
+		}
+	}
+
+	/* Try the local version of the mapping file */
+	szHome = szGetHomeDirectory();
+	if (strlen(szHome) + tFilenameLen <
+	    sizeof(szMappingFile) -
+	    sizeof(ANTIWORD_DIR) -
+	    2 * sizeof(FILE_SEPARATOR)) {
+		sprintf(szMappingFile,
+			"%s" FILE_SEPARATOR ANTIWORD_DIR FILE_SEPARATOR "%s%s",
+			szHome, szLeafname, szSuffix);
+		DBG_MSG(szMappingFile);
+		pFile = fopen(szMappingFile, "r");
+		if (pFile != NULL) {
+			return pFile;
+		}
+	} else {
+		werr(0, "Local mappingfilename too long, ignored");
+	}
+
+	/* Try the global version of the mapping file */
+	if (tFilenameLen <
+	    sizeof(szMappingFile) -
+	    sizeof(GLOBAL_ANTIWORD_DIR) -
+	    sizeof(FILE_SEPARATOR)) {
+		sprintf(szMappingFile,
+			GLOBAL_ANTIWORD_DIR FILE_SEPARATOR "%s%s",
+			szLeafname, szSuffix);
+		DBG_MSG(szMappingFile);
+		pFile = fopen(szMappingFile, "r");
+		if (pFile != NULL) {
+			return pFile;
+		}
+	} else {
+		werr(0, "Global mappingfilename too long, ignored");
+	}
+	werr(0, "I can't open your mapping file (%s%s)\n"
+		"It is not in '%s" FILE_SEPARATOR ANTIWORD_DIR "' nor in '"
+		GLOBAL_ANTIWORD_DIR "'.", szLeafname, szSuffix, szHome);
+	return NULL;
+#endif /* __riscos */
+} /* end of pOpenCharacterMappingFile */
+
+/*
+ * vCloseCharacterMappingFile - close the mapping file
+ */
+static void
+vCloseCharacterMappingFile(FILE *pFile)
+{
+	(void)fclose(pFile);
+} /* end of pCloseCharacterMappingFile */
+
+
 /*
  * iReadOptions - read options
  *
@@ -99,15 +286,13 @@ iReadOptions(int argc, char **argv)
 #else
 	extern	char	*optarg;
 	extern int	optind;
-	const papersize_type	*pPaperSize;
-	const char	*szHome, *szAntiword;
 	char	*pcChar, *szTmp;
 	int	iChar;
-	BOOL	bFound;
-	char	szLeafname[32+1];
-	char	szMappingFile[PATH_MAX+1];
+	char	szLeafname[LEAFNAME_SIZE];
 #endif /* __riscos */
+	FILE	*pCharacterMappingFile;
 	int	iTmp;
+	BOOL	bSuccess;
 
 	DBG_MSG("iReadOptions");
 
@@ -175,10 +360,14 @@ iReadOptions(int argc, char **argv)
 		werr(0, "Alphabet '%d' is not supported", iAlphabet);
 		return -1;
 	}
-	if (bReadCharacterMappingTable(szAlphabet)) {
-		return 1;
+	pCharacterMappingFile = pOpenCharacterMappingFile(szAlphabet);
+	if (pCharacterMappingFile != NULL) {
+		bSuccess = bReadCharacterMappingTable(pCharacterMappingFile);
+		vCloseCharacterMappingFile(pCharacterMappingFile);
+	} else {
+		bSuccess = FALSE;
 	}
-	return -1;
+	return bSuccess ? 1 : -1;
 #else
 /* Environment */
 	szTmp = getenv("COLUMNS");
@@ -196,19 +385,23 @@ iReadOptions(int argc, char **argv)
 			DBG_DEC(tOptionsCurr.iParagraphBreak);
 		}
 	}
-	if (is_locale_utf8()) {
-		tOptionsCurr.eEncoding = encoding_utf8;
-		strcpy(szLeafname, MAPPING_FILE_DEFAULT_8);
-	} else {
-		tOptionsCurr.eEncoding = encoding_iso_8859_1;
-		strcpy(szLeafname, MAPPING_FILE_DEFAULT_1);
-	}
+	strncpy(szLeafname, szGetDefaultMappingFile(), sizeof(szLeafname) - 1);
+	szLeafname[sizeof(szLeafname) - 1] = '\0';
 /* Command line */
-	while ((iChar = getopt(argc, argv, "Lhi:m:p:stw:x:")) != -1) {
+	while ((iChar = getopt(argc, argv, "La:fhi:m:p:rstw:x:")) != -1) {
 		switch (iChar) {
 		case 'L':
 			tOptionsCurr.bUseLandscape = TRUE;
 			break;
+		case 'a':
+			if (!bCorrectPapersize(optarg, conversion_pdf)) {
+				werr(0, "-a without a valid papersize");
+				return -1;
+			}
+			break;
+		case 'f':
+			tOptionsCurr.eConversionType = conversion_fmt_text;
+			break;
 		case 'h':
 			return 0;
 		case 'i':
@@ -243,38 +436,16 @@ iReadOptions(int argc, char **argv)
 			strncpy(szLeafname, optarg, sizeof(szLeafname) - 1);
 			szLeafname[sizeof(szLeafname) - 1] = '\0';
 			DBG_MSG(szLeafname);
-			if (STRCEQ(szLeafname, MAPPING_FILE_DEFAULT_8)) {
-				tOptionsCurr.eEncoding = encoding_utf8;
-			} else if (STRCEQ(szLeafname, MAPPING_FILE_DEFAULT_2)) {
-				tOptionsCurr.eEncoding = encoding_iso_8859_2;
-			} else {
-				tOptionsCurr.eEncoding = encoding_iso_8859_1;
-			}
-			DBG_DEC(tOptionsCurr.eEncoding);
 			break;
 		case 'p':
-			bFound = FALSE;
-			for (pPaperSize = atPaperSizes;
-			     pPaperSize->szName[0] != '\0';
-			     pPaperSize++) {
-				if (!STREQ(pPaperSize->szName,  optarg)) {
-					continue;
-				}
-				DBG_DEC(pPaperSize->usWidth);
-				DBG_DEC(pPaperSize->usHeight);
-				tOptionsCurr.eConversionType = conversion_ps;
-				tOptionsCurr.iPageHeight =
-						(int)pPaperSize->usHeight;
-				tOptionsCurr.iPageWidth =
-						(int)pPaperSize->usWidth;
-				bFound = TRUE;
-				break;
-			}
-			if (!bFound) {
+			if (!bCorrectPapersize(optarg, conversion_ps)) {
 				werr(0, "-p without a valid papersize");
 				return -1;
 			}
 			break;
+		case 'r':
+			tOptionsCurr.bRemoveRemovedText = FALSE;
+			break;
 		case 's':
 			tOptionsCurr.bHideHiddenText = FALSE;
 			break;
@@ -297,7 +468,7 @@ iReadOptions(int argc, char **argv)
 			if (STREQ(optarg, "db")) {
 				tOptionsCurr.iParagraphBreak = 0;
 				tOptionsCurr.eConversionType = conversion_xml;
-				tOptionsCurr.eEncoding = encoding_utf8;
+				strcpy(szLeafname, MAPPING_FILE_UTF_8);
 			} else {
 				werr(0, "-x %s is not supported", optarg);
 				return -1;
@@ -308,15 +479,33 @@ iReadOptions(int argc, char **argv)
 		}
 	}
 
+	tOptionsCurr.eEncoding = eMappingFile2Encoding(szLeafname);
+	DBG_DEC(tOptionsCurr.eEncoding);
+
 	if (tOptionsCurr.eConversionType == conversion_ps &&
-	    tOptionsCurr.eEncoding == encoding_utf8) {
+	    tOptionsCurr.eEncoding == encoding_utf_8) {
 		werr(0,
 		"The combination PostScript and UTF-8 is not supported");
 		return -1;
 	}
 
-	if (tOptionsCurr.eConversionType == conversion_ps) {
-		/* PostScript mode */
+	if (tOptionsCurr.eConversionType == conversion_pdf &&
+	    tOptionsCurr.eEncoding == encoding_utf_8) {
+		werr(0,
+		"The combination PDF and UTF-8 is not supported");
+		return -1;
+	}
+
+	if (tOptionsCurr.eConversionType == conversion_pdf &&
+	    tOptionsCurr.eEncoding == encoding_cyrillic) {
+		werr(0,
+		"The combination PDF and Cyrillic is not supported");
+		return -1;
+	}
+
+	if (tOptionsCurr.eConversionType == conversion_ps ||
+	    tOptionsCurr.eConversionType == conversion_pdf) {
+		/* PostScript or PDF mode */
 		if (tOptionsCurr.bUseLandscape) {
 			/* Swap the page height and width */
 			iTmp = tOptionsCurr.iPageHeight;
@@ -331,58 +520,14 @@ iReadOptions(int argc, char **argv)
 		DBG_DEC(tOptionsCurr.iParagraphBreak);
 	}
 
-	/* Try the environment version of the mapping file */
-	szAntiword = szGetAntiwordDirectory();
-	if (szAntiword != NULL && szAntiword[0] != '\0') {
-	    if (strlen(szAntiword) + strlen(szLeafname) <
-		sizeof(szMappingFile) -
-		sizeof(FILE_SEPARATOR)) {
-			sprintf(szMappingFile,
-				"%s" FILE_SEPARATOR "%s",
-				szAntiword, szLeafname);
-			DBG_MSG(szMappingFile);
-			if (bReadCharacterMappingTable(szMappingFile)) {
-				return optind;
-			}
-		} else {
-			werr(0, "Environment mappingfilename ignored");
-		}
-	}
-	/* Try the local version of the mapping file */
-	szHome = szGetHomeDirectory();
-	if (strlen(szHome) + strlen(szLeafname) <
-	    sizeof(szMappingFile) -
-	    sizeof(ANTIWORD_DIR) -
-	    2 * sizeof(FILE_SEPARATOR)) {
-		sprintf(szMappingFile,
-			"%s" FILE_SEPARATOR ANTIWORD_DIR FILE_SEPARATOR "%s",
-			szHome, szLeafname);
-		DBG_MSG(szMappingFile);
-		if (bReadCharacterMappingTable(szMappingFile)) {
-			return optind;
-		}
+	pCharacterMappingFile = pOpenCharacterMappingFile(szLeafname);
+	if (pCharacterMappingFile != NULL) {
+		bSuccess = bReadCharacterMappingTable(pCharacterMappingFile);
+		vCloseCharacterMappingFile(pCharacterMappingFile);
 	} else {
-		werr(0, "Local mappingfilename too long, ignored");
+		bSuccess = FALSE;
 	}
-	/* Try the global version of the mapping file */
-	if (strlen(szLeafname) <
-	    sizeof(szMappingFile) -
-	    sizeof(GLOBAL_ANTIWORD_DIR) -
-	    sizeof(FILE_SEPARATOR)) {
-		sprintf(szMappingFile,
-			GLOBAL_ANTIWORD_DIR FILE_SEPARATOR "%s",
-			szLeafname);
-		DBG_MSG(szMappingFile);
-		if (bReadCharacterMappingTable(szMappingFile)) {
-			return optind;
-		}
-	} else {
-		werr(0, "Global mappingfilename too long, ignored");
-	}
-	werr(0, "I can't open your mapping file (%s)\n"
-		"It is not in '%s" FILE_SEPARATOR ANTIWORD_DIR "' nor in '"
-		GLOBAL_ANTIWORD_DIR "'.", szLeafname, szHome);
-	return -1;
+	return bSuccess ? optind : -1;
 #endif /* __riscos */
 } /* end of iReadOptions */
 
@@ -407,7 +552,7 @@ vWriteOptions(void)
 	FILE	*pFile;
 	char	*szOptionsFile;
 
-	DBG_MSG("vWriteOptions");
+	TRACE_MSG("vWriteOptions");
 
 	szOptionsFile = getenv("AntiWord$ChoicesSave");
 	if (szOptionsFile == NULL) {
@@ -443,9 +588,9 @@ vWriteOptions(void)
  * vChoicesOpenAction - action to be taken when the Choices window opens
  */
 void
-vChoicesOpenAction(wimp_w tWindow)
+vChoicesOpenAction(window_handle tWindow)
 {
-	DBG_MSG("vChoicesOpenAction");
+	TRACE_MSG("vChoicesOpenAction");
 
 	tOptionsTemp = tOptionsCurr;
 	if (tOptionsTemp.iParagraphBreak == 0) {
@@ -483,15 +628,16 @@ vChoicesOpenAction(wimp_w tWindow)
 	}
 	vUpdateWriteableNumber(tWindow,
 		CHOICES_SCALE_WRITEABLE, tOptionsTemp.iScaleFactor);
+	TRACE_MSG("end of vChoicesOpenAction");
 } /* end of vChoicesOpenAction */
 
 /*
  * vDefaultButtonAction - action when the default button is clicked
  */
 static void
-vDefaultButtonAction(wimp_w tWindow)
+vDefaultButtonAction(window_handle tWindow)
 {
-	DBG_MSG("vDefaultButtonAction");
+	TRACE_MSG("vDefaultButtonAction");
 
 	tOptionsTemp = tOptionsDefault;
 	vUpdateRadioButton(tWindow, CHOICES_BREAK_BUTTON, TRUE);
@@ -520,7 +666,7 @@ vDefaultButtonAction(wimp_w tWindow)
 static void
 vApplyButtonAction(void)
 {
-	DBG_MSG("vApplyButtonAction");
+	TRACE_MSG("vApplyButtonAction");
 
 	tOptionsCurr = tOptionsTemp;
 } /* end of vApplyButtonAction */
@@ -531,7 +677,7 @@ vApplyButtonAction(void)
 static void
 vSaveButtonAction(void)
 {
-	DBG_MSG("vSaveButtonAction");
+	TRACE_MSG("vSaveButtonAction");
 
 	vApplyButtonAction();
 	vWriteOptions();
@@ -541,7 +687,7 @@ vSaveButtonAction(void)
  * vSetParagraphBreak - set the paragraph break to the given number
  */
 static void
-vSetParagraphBreak(wimp_w tWindow, int iNumber)
+vSetParagraphBreak(window_handle tWindow, int iNumber)
 {
 	tOptionsTemp.iParagraphBreak = iNumber;
 	if (tOptionsTemp.iParagraphBreak == 0) {
@@ -556,7 +702,7 @@ vSetParagraphBreak(wimp_w tWindow, int iNumber)
  * vChangeParagraphBreak - change the paragraph break with the given number
  */
 static void
-vChangeParagraphBreak(wimp_w tWindow, int iNumber)
+vChangeParagraphBreak(window_handle tWindow, int iNumber)
 {
 	int	iTmp;
 
@@ -575,7 +721,7 @@ vChangeParagraphBreak(wimp_w tWindow, int iNumber)
  * vChangeAutofiletype - invert the permission to autofiletype
  */
 static void
-vChangeAutofiletype(wimp_w tWindow)
+vChangeAutofiletype(window_handle tWindow)
 {
 	tOptionsTemp.bAutofiletypeAllowed =
 				!tOptionsTemp.bAutofiletypeAllowed;
@@ -588,7 +734,7 @@ vChangeAutofiletype(wimp_w tWindow)
  * vChangeHiddenText - invert the hide/show hidden text
  */
 static void
-vChangeHiddenText(wimp_w tWindow)
+vChangeHiddenText(window_handle tWindow)
 {
 	tOptionsTemp.bHideHiddenText = !tOptionsTemp.bHideHiddenText;
 	vUpdateRadioButton(tWindow,
@@ -613,7 +759,7 @@ vUseFontsImages(BOOL bUseOutlineFonts, BOOL bShowImages)
  * vSetScaleFactor - set the scale factor to the given number
  */
 static void
-vSetScaleFactor(wimp_w tWindow, int iNumber)
+vSetScaleFactor(window_handle tWindow, int iNumber)
 {
   	tOptionsTemp.iScaleFactor = iNumber;
 	vUpdateWriteableNumber(tWindow,
@@ -625,7 +771,7 @@ vSetScaleFactor(wimp_w tWindow, int iNumber)
  * vChangeScaleFactor - change the scale factor with the given number
  */
 static void
-vChangeScaleFactor(wimp_w tWindow, int iNumber)
+vChangeScaleFactor(window_handle tWindow, int iNumber)
 {
 	int	iTmp;
 
@@ -643,24 +789,30 @@ vChangeScaleFactor(wimp_w tWindow, int iNumber)
 /*
  * bChoicesMouseClick - handle a mouse click in the Choices window
  */
-void
-vChoicesMouseClick(wimp_mousestr *m)
+BOOL
+bChoicesMouseClick(event_pollblock *pEvent, void *pvReference)
 {
-	wimp_i	tAction;
-	BOOL	bCloseWindow, bLeft, bRight;
+	icon_handle	tAction;
+	mouse_block	*pMouse;
+	BOOL		bCloseWindow;
 
-	bLeft = (m->bbits & wimp_BLEFT) == wimp_BLEFT;
-	bRight = (m->bbits & wimp_BRIGHT) == wimp_BRIGHT;
-	if (!bLeft && !bRight) {
-		DBG_HEX(m->bbits);
-		return;
+	TRACE_MSG("bChoicesMouseClick");
+
+	fail(pEvent == NULL);
+	fail(pEvent->type != event_CLICK);
+
+	pMouse = &pEvent->data.mouse;
+	if (!pMouse->button.data.select && !pMouse->button.data.adjust) {
+		/* Not handled here */
+		DBG_HEX(pMouse->button.value);
+		return FALSE;
 	}
 
 	/* Which action should be taken */
-	tAction = m->i;
-	if (bRight) {
-	  	/* The right button reverses the direction */
-		switch (m->i) {
+	tAction = pMouse->icon;
+	if (pMouse->button.data.adjust) {
+	  	/* The adjust button reverses the direction */
+		switch (pMouse->icon) {
 		case CHOICES_BREAK_UP_BUTTON:
 			tAction = CHOICES_BREAK_DOWN_BUTTON;
 			break;
@@ -682,7 +834,7 @@ vChoicesMouseClick(wimp_mousestr *m)
 	bCloseWindow = FALSE;
 	switch (tAction) {
 	case CHOICES_DEFAULT_BUTTON:
-		vDefaultButtonAction(m->w);
+		vDefaultButtonAction(pMouse->window);
 		break;
 	case CHOICES_SAVE_BUTTON:
 		vSaveButtonAction();
@@ -695,22 +847,22 @@ vChoicesMouseClick(wimp_mousestr *m)
 		bCloseWindow = TRUE;
 		break;
 	case CHOICES_BREAK_BUTTON:
-		vSetParagraphBreak(m->w, DEFAULT_SCREEN_WIDTH);
+		vSetParagraphBreak(pMouse->window, DEFAULT_SCREEN_WIDTH);
 		break;
 	case CHOICES_BREAK_UP_BUTTON:
-		vChangeParagraphBreak(m->w, 1);
+		vChangeParagraphBreak(pMouse->window, 1);
 		break;
 	case CHOICES_BREAK_DOWN_BUTTON:
-		vChangeParagraphBreak(m->w, -1);
+		vChangeParagraphBreak(pMouse->window, -1);
 		break;
 	case CHOICES_NO_BREAK_BUTTON:
-		vSetParagraphBreak(m->w, 0);
+		vSetParagraphBreak(pMouse->window, 0);
 		break;
 	case CHOICES_AUTOFILETYPE_BUTTON:
-		vChangeAutofiletype(m->w);
+		vChangeAutofiletype(pMouse->window);
 		break;
 	case CHOICES_HIDDEN_TEXT_BUTTON:
-		vChangeHiddenText(m->w);
+		vChangeHiddenText(pMouse->window);
 		break;
 	case CHOICES_WITH_IMAGES_BUTTON:
 		vUseFontsImages(TRUE, TRUE);
@@ -722,40 +874,51 @@ vChoicesMouseClick(wimp_mousestr *m)
 		vUseFontsImages(FALSE, FALSE);
 		break;
 	case CHOICES_SCALE_UP_BUTTON:
-		vChangeScaleFactor(m->w, 5);
+		vChangeScaleFactor(pMouse->window, 5);
 		break;
 	case CHOICES_SCALE_DOWN_BUTTON:
-		vChangeScaleFactor(m->w, -5);
+		vChangeScaleFactor(pMouse->window, -5);
 		break;
 	default:
-		DBG_DEC(m->i);
+		DBG_DEC(pMouse->icon);
 		break;
 	}
 	if (bCloseWindow) {
-		wimpt_noerr(wimp_close_wind(m->w));
+		Error_CheckFatal(Wimp_CloseWindow(pMouse->window));
 	}
-} /* end of vChoicesMouseClick */
+	return TRUE;
+} /* end of bChoicesMouseClick */
 
-void
-vChoicesKeyPressed(wimp_caretstr *c)
+/*
+ * bChoicesKeyPressed - handle a key in the Choices window
+ */
+BOOL
+bChoicesKeyPressed(event_pollblock *pEvent, void *pvReference)
 {
-	wimp_icon	tIcon;
+	icon_block	tIcon;
+	caret_block	*pCaret;
 	char		*pcChar;
 	int		iNumber;
 
-	DBG_MSG("vChoicesKeyPressed");
+	DBG_MSG("bChoicesKeyPressed");
 
-	fail(c == NULL);
+	fail(pEvent == NULL);
+	fail(pEvent->type != event_KEY);
 
-	wimpt_noerr(wimp_get_icon_info(c->w, c->i, &tIcon));
-	if ((tIcon.flags & (wimp_ITEXT|wimp_INDIRECT)) !=
-	    (wimp_ITEXT|wimp_INDIRECT)) {
-		werr(1, "Icon %d must be indirected text", (int)c->i);
-		return;
+	if (pEvent->data.key.code != '\r') {
+		Error_CheckFatal(Wimp_ProcessKey(pEvent->data.key.code));
+		return TRUE;
+	}
+
+	pCaret = &pEvent->data.key.caret;
+
+	Error_CheckFatal(Wimp_GetIconState(pCaret->window, pCaret->icon, &tIcon));
+	if (!tIcon.flags.data.text || !tIcon.flags.data.indirected) {
+		werr(1, "Icon %d must be indirected text", (int)pCaret->icon);
 	}
 	iNumber = (int)strtol(tIcon.data.indirecttext.buffer, &pcChar, 10);
 
-	switch(c->i) {
+	switch(pCaret->icon) {
 	case CHOICES_BREAK_WRITEABLE:
 		if (*pcChar != '\0' && *pcChar != '\r') {
 			DBG_DEC(*pcChar);
@@ -765,7 +928,7 @@ vChoicesKeyPressed(wimp_caretstr *c)
 		} else if (iNumber > MAX_SCREEN_WIDTH) {
 			iNumber = MAX_SCREEN_WIDTH;
 		}
-		vSetParagraphBreak(c->w, iNumber);
+		vSetParagraphBreak(pCaret->window, iNumber);
 		break;
 	case CHOICES_SCALE_WRITEABLE:
 		if (*pcChar != '\0' && *pcChar != '\r') {
@@ -776,11 +939,12 @@ vChoicesKeyPressed(wimp_caretstr *c)
 		} else if (iNumber > MAX_SCALE_FACTOR) {
 			iNumber = MAX_SCALE_FACTOR;
 		}
-		vSetScaleFactor(c->w, iNumber);
+		vSetScaleFactor(pCaret->window, iNumber);
 		break;
 	default:
-		DBG_DEC(c->i);
+		DBG_DEC(pCaret->icon);
 		break;
 	}
-} /* end of vChoicesKeyPressed */
+	return TRUE;
+} /* end of bChoicesKeyPressed */
 #endif /* __riscos */
