@@ -1,6 +1,6 @@
 /*
  * prop2.c
- * Copyright (C) 2002 A.J. van Os; Released under GPL
+ * Copyright (C) 2002-2005 A.J. van Os; Released under GPL
  *
  * Description:
  * Read the property information from a WinWord 1 or 2 file
@@ -10,6 +10,8 @@
 #include "antiword.h"
 
 
+#define MAX_FILESIZE		0x2000000UL	/* 32 Mb */
+
 /*
  * iGet2InfoLength - the length of the information for WinWord 1/2 files
  */
@@ -53,6 +55,45 @@ iGet2InfoLength(int iByteNbr, const UCHAR *aucGrpprl)
 	}
 } /* end of iGet2InfoLength */
 
+/*
+ * Build the lists with Document Property Information for WinWord 1/2 files
+ */
+void
+vGet2DopInfo(FILE *pFile, const UCHAR *aucHeader)
+{
+	document_block_type	tDocument;
+	UCHAR	*aucBuffer;
+	ULONG	ulBeginDocpInfo, ulTmp;
+	size_t	tDocpInfoLen;
+	USHORT	usTmp;
+
+	ulBeginDocpInfo = ulGetLong(0x112, aucHeader); /* fcDop */
+	DBG_HEX(ulBeginDocpInfo);
+	tDocpInfoLen = (size_t)usGetWord(0x116, aucHeader); /* cbDop */
+	DBG_DEC(tDocpInfoLen);
+	if (tDocpInfoLen < 28) {
+		DBG_MSG("No Document information");
+		return;
+	}
+
+	aucBuffer = xmalloc(tDocpInfoLen);
+	if (!bReadBytes(aucBuffer, tDocpInfoLen, ulBeginDocpInfo, pFile)) {
+		aucBuffer = xfree(aucBuffer);
+		return;
+	}
+
+	usTmp = usGetWord(0x00, aucBuffer);
+	tDocument.ucHdrFtrSpecification = (UCHAR)(usTmp >> 8); /* grpfIhdt */
+	tDocument.usDefaultTabWidth = usGetWord(0x0a, aucBuffer); /* dxaTab */
+	ulTmp = ulGetLong(0x14, aucBuffer); /* dttmCreated */
+	tDocument.tCreateDate = tConvertDTTM(ulTmp);
+	ulTmp = ulGetLong(0x18, aucBuffer); /* dttmRevised */
+	tDocument.tRevisedDate = tConvertDTTM(ulTmp);
+	vCreateDocumentInfoList(&tDocument);
+
+	aucBuffer = xfree(aucBuffer);
+} /* end of vGet2DopInfo */
+
 /*
  * Fill the section information block with information
  * from a WinWord 1/2 file.
@@ -69,7 +110,6 @@ vGet2SectionInfo(const UCHAR *aucGrpprl, size_t tBytes,
 
 	iFodoOff = 0;
 	while (tBytes >= (size_t)iFodoOff + 1) {
-		iInfoLen = 0;
 		switch (ucGetByte(iFodoOff, aucGrpprl)) {
 		case 117:	/* bkc */
 			ucTmp = ucGetByte(iFodoOff + 1, aucGrpprl);
@@ -80,13 +120,15 @@ vGet2SectionInfo(const UCHAR *aucGrpprl, size_t tBytes,
 			usCcol = 1 + usGetWord(iFodoOff + 1, aucGrpprl);
 			DBG_DEC(usCcol);
 			break;
+		case 128:	/* grpfIhdt */
+			pSection->ucHdrFtrSpecification =
+					ucGetByte(iFodoOff + 1, aucGrpprl);
+			break;
 		default:
 			break;
 		}
-		if (iInfoLen <= 0) {
-			iInfoLen = iGet2InfoLength(iFodoOff, aucGrpprl);
-			fail(iInfoLen <= 0);
-		}
+		iInfoLen = iGet2InfoLength(iFodoOff, aucGrpprl);
+		fail(iInfoLen <= 0);
 		iFodoOff += iInfoLen;
 	}
 } /* end of vGet2SectionInfo */
@@ -98,15 +140,16 @@ void
 vGet2SepInfo(FILE *pFile, const UCHAR *aucHeader)
 {
 	section_block_type	tSection;
-	ULONG	*aulSectPage, *aulTextOffset;
+	ULONG	*aulSectPage, *aulCharPos;
 	UCHAR	*aucBuffer, *aucFpage;
-	ULONG	ulBeginSectInfo;
-	size_t	tSectInfoLen, tOffset, tSize, tLen, tBytes;
-	int	iIndex;
+	ULONG	ulBeginOfText, ulTextOffset, ulBeginSectInfo;
+	size_t	tSectInfoLen, tIndex, tOffset, tLen, tBytes;
 	UCHAR	aucTmp[1];
 
 	fail(pFile == NULL || aucHeader == NULL);
 
+	ulBeginOfText = ulGetLong(0x18, aucHeader); /* fcMin */
+	NO_DBG_HEX(ulBeginOfText);
 	ulBeginSectInfo = ulGetLong(0x7c, aucHeader); /* fcPlcfsed */
 	DBG_HEX(ulBeginSectInfo);
 	tSectInfoLen = (size_t)usGetWord(0x80, aucHeader); /* cbPlcfsed */
@@ -125,39 +168,41 @@ vGet2SepInfo(FILE *pFile, const UCHAR *aucHeader)
 
 	/* Read the Section Descriptors */
 	tLen = (tSectInfoLen - 4) / 10;
-	tSize = tLen * sizeof(ULONG);
 	/* Save the section offsets */
-	aulTextOffset = xmalloc(tSize);
-	for (iIndex = 0, tOffset = 0;
-	     iIndex < (int)tLen;
-	     iIndex++, tOffset += 4) {
-		aulTextOffset[iIndex] = ulGetLong(tOffset, aucBuffer);
+	aulCharPos = xcalloc(tLen, sizeof(ULONG));
+	for (tIndex = 0, tOffset = 0;
+	     tIndex < tLen;
+	     tIndex++, tOffset += 4) {
+		ulTextOffset = ulGetLong(tOffset, aucBuffer);
+		NO_DBG_HEX(ulTextOffset);
+		aulCharPos[tIndex] = ulBeginOfText + ulTextOffset;
+		NO_DBG_HEX(aulCharPos[tIndex]);
 	}
 	/* Save the Sepx offsets */
-	aulSectPage = xmalloc(tSize);
-	for (iIndex = 0, tOffset = (tLen + 1) * 4;
-	     iIndex < (int)tLen;
-	     iIndex++, tOffset += 6) {
-		aulSectPage[iIndex] = ulGetLong(tOffset + 2, aucBuffer);
-		NO_DBG_HEX(aulSectPage[iIndex]); /* fcSepx */
+	aulSectPage = xcalloc(tLen, sizeof(ULONG));
+	for (tIndex = 0, tOffset = (tLen + 1) * 4;
+	     tIndex < tLen;
+	     tIndex++, tOffset += 6) {
+		aulSectPage[tIndex] = ulGetLong(tOffset + 2, aucBuffer);
+		NO_DBG_HEX(aulSectPage[tIndex]); /* fcSepx */
 	}
 	aucBuffer = xfree(aucBuffer);
 
 	/* Read the Section Properties */
-	for (iIndex = 0; iIndex < (int)tLen; iIndex++) {
-		if (aulSectPage[iIndex] == FC_INVALID) {
-			vDefault2SectionInfoList(aulTextOffset[iIndex]);
+	for (tIndex = 0; tIndex < tLen; tIndex++) {
+		if (aulSectPage[tIndex] == FC_INVALID) {
+			vDefault2SectionInfoList(aulCharPos[tIndex]);
 			continue;
 		}
 		/* Get the number of bytes to read */
-		if (!bReadBytes(aucTmp, 1, aulSectPage[iIndex], pFile)) {
+		if (!bReadBytes(aucTmp, 1, aulSectPage[tIndex], pFile)) {
 			continue;
 		}
 		tBytes = 1 + (size_t)ucGetByte(0, aucTmp);
 		NO_DBG_DEC(tBytes);
 		/* Read the bytes */
 		aucFpage = xmalloc(tBytes);
-		if (!bReadBytes(aucFpage, tBytes, aulSectPage[iIndex], pFile)) {
+		if (!bReadBytes(aucFpage, tBytes, aulSectPage[tIndex], pFile)) {
 			aucFpage = xfree(aucFpage);
 			continue;
 		}
@@ -165,13 +210,58 @@ vGet2SepInfo(FILE *pFile, const UCHAR *aucHeader)
 		/* Process the bytes */
 		vGetDefaultSection(&tSection);
 		vGet2SectionInfo(aucFpage + 1, tBytes - 1, &tSection);
-		vAdd2SectionInfoList(&tSection, aulTextOffset[iIndex]);
+		vAdd2SectionInfoList(&tSection, aulCharPos[tIndex]);
 		aucFpage = xfree(aucFpage);
 	}
-	aulTextOffset = xfree(aulTextOffset);
+	aulCharPos = xfree(aulCharPos);
 	aulSectPage = xfree(aulSectPage);
 } /* end of vGet2SepInfo */
 
+/*
+ * Build the list with Header/Footer Information for WinWord 1/2 files
+ */
+void
+vGet2HdrFtrInfo(FILE *pFile, const UCHAR *aucHeader)
+{
+	ULONG	*aulCharPos;
+	UCHAR	*aucBuffer;
+	ULONG	ulHdrFtrOffset, ulBeginHdrFtrInfo;
+	size_t	tHdrFtrInfoLen, tIndex, tOffset, tLen;
+
+	fail(pFile == NULL || aucHeader == NULL);
+
+	ulBeginHdrFtrInfo = ulGetLong(0x9a, aucHeader); /* fcPlcfhdd */
+	NO_DBG_HEX(ulBeginHdrFtrInfo);
+	tHdrFtrInfoLen = (size_t)usGetWord(0x9e, aucHeader); /* cbPlcfhdd */
+	NO_DBG_DEC(tHdrFtrInfoLen);
+	if (tHdrFtrInfoLen < 8) {
+		DBG_DEC_C(tHdrFtrInfoLen != 0, tHdrFtrInfoLen);
+		return;
+	}
+
+	aucBuffer = xmalloc(tHdrFtrInfoLen);
+	if (!bReadBytes(aucBuffer, tHdrFtrInfoLen, ulBeginHdrFtrInfo, pFile)) {
+		aucBuffer = xfree(aucBuffer);
+		return;
+	}
+	NO_DBG_PRINT_BLOCK(aucBuffer, tHdrFtrInfoLen);
+
+	tLen = tHdrFtrInfoLen / 4 - 1;
+	/* Save the header/footer offsets */
+	aulCharPos = xcalloc(tLen, sizeof(ULONG));
+	for (tIndex = 0, tOffset = 0;
+	     tIndex < tLen;
+	     tIndex++, tOffset += 4) {
+		ulHdrFtrOffset = ulGetLong(tOffset, aucBuffer);
+		NO_DBG_HEX(ulHdrFtrOffset);
+		aulCharPos[tIndex] = ulHdrFtrOffset2CharPos(ulHdrFtrOffset);
+		NO_DBG_HEX(aulCharPos[tIndex]);
+	}
+	vCreat2HdrFtrInfoList(aulCharPos, tLen);
+	aulCharPos = xfree(aulCharPos);
+	aucBuffer = xfree(aucBuffer);
+} /* end of vGet2HdrFtrInfo */
+
 /*
  * Translate the rowinfo to a member of the row_info enumeration
  */
@@ -182,6 +272,7 @@ eGet2RowInfo(int iFodo,
 	int	iFodoOff, iInfoLen;
 	int	iIndex, iSize, iCol;
 	int	iPosCurr, iPosPrev;
+	USHORT	usTmp;
 	BOOL	bFound24_0, bFound24_1, bFound25_0, bFound25_1, bFound154;
 
 	fail(iFodo < 0 || aucGrpprl == NULL || pRow == NULL);
@@ -209,6 +300,86 @@ eGet2RowInfo(int iFodo,
 				bFound25_0 = TRUE;
 			}
 			break;
+		case 30:	/* brcTop10 */
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			usTmp &= 0x01ff;
+			NO_DBG_DEC(usTmp >> 6);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_TOP;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_TOP;
+			}
+			break;
+		case 31:	/* brcLeft10 */
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			usTmp &= 0x01ff;
+			NO_DBG_DEC(usTmp >> 6);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_LEFT;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_LEFT;
+			}
+			break;
+		case 32:	/* brcBottom10 */
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			usTmp &= 0x01ff;
+			NO_DBG_DEC(usTmp >> 6);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_BOTTOM;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_BOTTOM;
+			}
+			break;
+		case 33:	/* brcRight10 */
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			usTmp &= 0x01ff;
+			NO_DBG_DEC(usTmp >> 6);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_RIGHT;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_RIGHT;
+			}
+			break;
+		case 38:	/* brcTop */
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			usTmp &= 0x0018;
+			NO_DBG_DEC(usTmp >> 3);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_TOP;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_TOP;
+			}
+			break;
+		case 39:	/* brcLeft */
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			usTmp &= 0x0018;
+			NO_DBG_DEC(usTmp >> 3);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_LEFT;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_LEFT;
+			}
+			break;
+		case 40:	/* brcBottom */
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			usTmp &= 0x0018;
+			NO_DBG_DEC(usTmp >> 3);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_BOTTOM;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_BOTTOM;
+			}
+			break;
+		case 41:	/* brcRight */
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			usTmp &= 0x0018;
+			NO_DBG_DEC(usTmp >> 3);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_RIGHT;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_RIGHT;
+			}
+			break;
 		case 152:	/* cDefTable10 */
 		case 154:	/* cDefTable */
 			iSize = (int)usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
@@ -225,6 +396,7 @@ eGet2RowInfo(int iFodo,
 				DBG_DEC(iCol);
 				DBG_DEC(iBytes);
 				DBG_DEC(iFodoOff);
+				DBG_DEC(ucGetByte(iFodo + iFodoOff, aucGrpprl));
 				iInfoLen = 1;
 				break;
 			}
@@ -233,7 +405,6 @@ eGet2RowInfo(int iFodo,
 				werr(1, "The number of columns is corrupt");
 			}
 			pRow->ucNumberOfColumns = (UCHAR)iCol;
-			pRow->iColumnWidthSum = 0;
 			iPosPrev = (int)(short)usGetWord(
 					iFodo + iFodoOff + 4,
 					aucGrpprl);
@@ -243,8 +414,6 @@ eGet2RowInfo(int iFodo,
 					aucGrpprl);
 				pRow->asColumnWidth[iIndex] =
 						(short)(iPosCurr - iPosPrev);
-				pRow->iColumnWidthSum +=
-					pRow->asColumnWidth[iIndex];
 				iPosPrev = iPosCurr;
 			}
 			bFound154 = TRUE;
@@ -310,21 +479,12 @@ vGet2StyleInfo(int iFodo,
 			break;
 		case  13:	/* nLvlAnm */
 			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
-			pStyle->ucListLevel = ucTmp;
-#if 0
-			if (iTmp == 0) {
-				pStyle->bNoNumbering = TRUE;
-			}
-			if (iTmp >= 12) {
-				pStyle->bNumPause = TRUE;
-			}
-#else
-			if ((int)ucTmp == 0 || (int)ucTmp >= 12) {
-				pStyle->bNumPause = TRUE;
-			}
-#endif
+			pStyle->ucNumLevel = ucTmp;
+			pStyle->bNumPause =
+				eGetNumType(ucTmp) == level_type_pause;
 			break;
 		case  15:	/* ChgTabsPapx */
+		case  23:	/* ChgTabs */
 			iTmp = (int)ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			if (iTmp < 2) {
 				iInfoLen = 1;
@@ -365,6 +525,11 @@ vGet2StyleInfo(int iFodo,
 			NO_DBG_DEC(sTmp);
 			NO_DBG_DEC(pStyle->sLeftIndent);
 			break;
+		case  19:	/* dxaLeft1 */
+			pStyle->sLeftIndent1 = (short)usGetWord(
+					iFodo + iFodoOff + 1, aucGrpprl);
+			NO_DBG_DEC(pStyle->sLeftIndent1);
+			break;
 		case  21:	/* dyaBefore */
 			pStyle->usBeforeIndent = usGetWord(
 					iFodo + iFodoOff + 1, aucGrpprl);
@@ -425,8 +590,7 @@ vGet2PapInfo(FILE *pFile, const UCHAR *aucHeader)
 	NO_DBG_PRINT_BLOCK(aucBuffer, tParfInfoLen);
 
 	tLen = (tParfInfoLen - 4) / 6;
-	tSize = tLen * sizeof(USHORT);
-	ausParfPage = xmalloc(tSize);
+	ausParfPage = xcalloc(tLen, sizeof(USHORT));
 	for (iIndex = 0, tOffset = (tLen + 1) * 4;
 	     iIndex < (int)tLen;
 	     iIndex++, tOffset += 2) {
@@ -466,6 +630,9 @@ vGet2PapInfo(FILE *pFile, const UCHAR *aucHeader)
 		iRun = (int)ucGetByte(0x1ff, aucFpage);
 		NO_DBG_DEC(iRun);
 		for (iIndex2 = 0; iIndex2 < iRun; iIndex2++) {
+			if ((iRun + 1) * 4 + iIndex2 * 1 >= BIG_BLOCK_SIZE) {
+				break;
+			}
 			NO_DBG_HEX(ulGetLong(iIndex2 * 4, aucFpage));
 			iFodo = 2 * (int)ucGetByte(
 				(iRun + 1) * 4 + iIndex2 * 1, aucFpage);
@@ -495,14 +662,14 @@ vGet2PapInfo(FILE *pFile, const UCHAR *aucHeader)
 				}
 				ulCharPosFirst = ulGetLong(
 						iIndex2 * 4, aucFpage);
-				DBG_HEX(ulCharPosFirst);
+				NO_DBG_HEX(ulCharPosFirst);
 				tRow.ulCharPosStart = ulCharPosFirst;
 				tRow.ulFileOffsetStart = ulCharPosFirst;
 				break;
 			case found_end_of_row:
 				ulCharPosLast = ulGetLong(
 						iIndex2 * 4, aucFpage);
-				DBG_HEX(ulCharPosLast);
+				NO_DBG_HEX(ulCharPosLast);
 				tRow.ulCharPosEnd = ulCharPosLast;
 				/* Add 1 for compatiblity with Word 6 and up */
 				tRow.ulFileOffsetEnd = ulCharPosLast + 1;
@@ -524,7 +691,6 @@ vGet2PapInfo(FILE *pFile, const UCHAR *aucHeader)
 /*
  * Fill the font information block with information
  * from a WinWord 1 file.
- * Returns TRUE when successful, otherwise FALSE
  */
 void
 vGet1FontInfo(int iFodo,
@@ -536,9 +702,11 @@ vGet1FontInfo(int iFodo,
 	UCHAR	aucChpx[12];
 
 	fail(iFodo < 0 || aucGrpprl == NULL || pFont == NULL);
-	fail(tBytes > sizeof(aucChpx));
 
-	NO_DBG_PRINT_BLOCK(aucGrpprl + iFodo, iBytes);
+	if (tBytes > sizeof(aucChpx)) {
+		NO_DBG_PRINT_BLOCK(aucGrpprl + iFodo, tBytes);
+		return;
+	}
 
 	/* Build the CHPX structure */
 	(void)memset(aucChpx, 0, sizeof(aucChpx));
@@ -546,27 +714,34 @@ vGet1FontInfo(int iFodo,
 
 	usTmp = usGetWord(0, aucChpx);
 	if ((usTmp & BIT(0)) != 0) {
-		pFont->ucFontstyle ^= FONT_BOLD;
+		pFont->usFontStyle ^= FONT_BOLD;
 	}
 	if ((usTmp & BIT(1)) != 0) {
-		pFont->ucFontstyle ^= FONT_ITALIC;
+		pFont->usFontStyle ^= FONT_ITALIC;
 	}
 	if ((usTmp & BIT(2)) != 0) {
-		pFont->ucFontstyle ^= FONT_STRIKE;
+		pFont->usFontStyle ^= FONT_STRIKE;
 	}
 	if ((usTmp & BIT(5)) != 0) {
-		pFont->ucFontstyle ^= FONT_SMALL_CAPITALS;
+		pFont->usFontStyle ^= FONT_SMALL_CAPITALS;
 	}
 	if ((usTmp & BIT(6)) != 0) {
-		pFont->ucFontstyle ^= FONT_CAPITALS;
+		pFont->usFontStyle ^= FONT_CAPITALS;
 	}
 	if ((usTmp & BIT(7)) != 0) {
-		pFont->ucFontstyle ^= FONT_HIDDEN;
+		pFont->usFontStyle ^= FONT_HIDDEN;
 	}
 
 	ucTmp = ucGetByte(5, aucChpx);
-	DBG_MSG_C(ucTmp != 0 && ucTmp < 128, "Superscript");
-	DBG_MSG_C(ucTmp >= 128, "Subscript");
+	if (ucTmp != 0) {
+		if (ucTmp < 128) {
+			pFont->usFontStyle |= FONT_SUPERSCRIPT;
+			DBG_MSG("Superscript");
+		} else {
+			pFont->usFontStyle |= FONT_SUBSCRIPT;
+			DBG_MSG("Subscript");
+		}
+	}
 
 	bIcoChange = (usTmp & BIT(10)) != 0;
 	bFtcChange = (usTmp & BIT(11)) != 0;
@@ -576,35 +751,35 @@ vGet1FontInfo(int iFodo,
 	if (bFtcChange) {
 		usTmp = usGetWord(2, aucChpx);
 		if (usTmp <= (USHORT)UCHAR_MAX) {
-			pFont->ucFontnumber = (UCHAR)usTmp;
+			pFont->ucFontNumber = (UCHAR)usTmp;
 		} else {
-			pFont->ucFontnumber = 0;
+			pFont->ucFontNumber = 0;
 		}
 	}
 
 	if (bHpsChange) {
-		pFont->sFontsize = (short)ucGetByte(4, aucChpx);
+		pFont->usFontSize = (USHORT)ucGetByte(4, aucChpx);
 	}
 
 	if (bIcoChange || bKulChange) {
 		usTmp = usGetWord(6, aucChpx);
 		if (bIcoChange) {
-			pFont->ucFontcolor = (UCHAR)((usTmp & 0x0f00) >> 8);
-			if (pFont->ucFontcolor <= 7) {
+			pFont->ucFontColor = (UCHAR)((usTmp & 0x0f00) >> 8);
+			if (pFont->ucFontColor <= 7) {
 				/* Add 1 for compatibility with Word 2 and up */
-				pFont->ucFontcolor++;
+				pFont->ucFontColor++;
 			} else {
-				DBG_DEC(pFont->ucFontcolor);
-				pFont->ucFontcolor = 0;
+				DBG_DEC(pFont->ucFontColor);
+				pFont->ucFontColor = 0;
 			}
 		}
 		if (bKulChange) {
 			usTmp = (usTmp & 0x7000) >> 12;
 			DBG_DEC_C(usTmp > 4, usTmp);
 			if (usTmp == 0) {
-				pFont->ucFontstyle &= ~FONT_UNDERLINE;
+				pFont->usFontStyle &= ~FONT_UNDERLINE;
 			} else {
-				pFont->ucFontstyle |= FONT_UNDERLINE;
+				pFont->usFontStyle |= FONT_UNDERLINE;
 			}
 		}
 	}
@@ -613,7 +788,6 @@ vGet1FontInfo(int iFodo,
 /*
  * Fill the font information block with information
  * from a WinWord 1/2 file.
- * Returns TRUE when successful, otherwise FALSE
  */
 void
 vGet2FontInfo(int iFodo,
@@ -625,9 +799,11 @@ vGet2FontInfo(int iFodo,
 	UCHAR	aucChpx[18];
 
 	fail(iFodo < 0 || aucGrpprl == NULL || pFont == NULL);
-	fail(tBytes > sizeof(aucChpx));
 
-	NO_DBG_PRINT_BLOCK(aucGrpprl + iFodo, tBytes);
+	if (tBytes > sizeof(aucChpx)) {
+		NO_DBG_PRINT_BLOCK(aucGrpprl + iFodo, tBytes);
+		return;
+	}
 
 	/* Build the CHPX structure */
 	(void)memset(aucChpx, 0, sizeof(aucChpx));
@@ -635,25 +811,25 @@ vGet2FontInfo(int iFodo,
 
 	usTmp = usGetWord(0, aucChpx);
 	if ((usTmp & BIT(0)) != 0) {
-		pFont->ucFontstyle ^= FONT_BOLD;
+		pFont->usFontStyle ^= FONT_BOLD;
 	}
 	if ((usTmp & BIT(1)) != 0) {
-		pFont->ucFontstyle ^= FONT_ITALIC;
+		pFont->usFontStyle ^= FONT_ITALIC;
 	}
 	if (usTmp & BIT(3)) {
-		pFont->ucFontstyle ^= FONT_MARKDEL;
+		pFont->usFontStyle ^= FONT_MARKDEL;
 	}
 	if ((usTmp & BIT(5)) != 0) {
-		pFont->ucFontstyle ^= FONT_SMALL_CAPITALS;
+		pFont->usFontStyle ^= FONT_SMALL_CAPITALS;
 	}
 	if ((usTmp & BIT(6)) != 0) {
-		pFont->ucFontstyle ^= FONT_CAPITALS;
+		pFont->usFontStyle ^= FONT_CAPITALS;
 	}
 	if ((usTmp & BIT(7)) != 0) {
-		pFont->ucFontstyle ^= FONT_HIDDEN;
+		pFont->usFontStyle ^= FONT_HIDDEN;
 	}
 	if (usTmp & BIT(10)) {
-		pFont->ucFontstyle ^= FONT_STRIKE;
+		pFont->usFontStyle ^= FONT_STRIKE;
 	}
 
 	ucTmp = ucGetByte(10, aucChpx);
@@ -674,37 +850,32 @@ vGet2FontInfo(int iFodo,
 	if (bFtcChange) {
 		usTmp = usGetWord(4, aucChpx);
 		if (usTmp <= (USHORT)UCHAR_MAX) {
-			pFont->ucFontnumber = (UCHAR)usTmp;
+			pFont->ucFontNumber = (UCHAR)usTmp;
 		} else {
-			pFont->ucFontnumber = 0;
+			pFont->ucFontNumber = 0;
 		}
 	}
 
 	if (bHpsChange) {
-		usTmp = usGetWord(6, aucChpx);
-		if (usTmp > (USHORT)SHRT_MAX) {
-			pFont->sFontsize = SHRT_MAX;
-		} else {
-			pFont->sFontsize = (short)usTmp;
-		}
+		pFont->usFontSize = usGetWord(6, aucChpx);
 	}
 
 	if (bIcoChange || bKulChange) {
 		ucTmp = ucGetByte(9, aucChpx);
 		if (bIcoChange) {
-			pFont->ucFontcolor = ucTmp & 0x1f;
-			if (pFont->ucFontcolor > 16) {
-				DBG_DEC(pFont->ucFontcolor);
-				pFont->ucFontcolor = 0;
+			pFont->ucFontColor = ucTmp & 0x1f;
+			if (pFont->ucFontColor > 16) {
+				DBG_DEC(pFont->ucFontColor);
+				pFont->ucFontColor = 0;
 			}
 		}
 		if (bKulChange) {
 			ucTmp = (ucTmp & 0xe0) >> 5;
 			DBG_DEC_C(ucTmp > 4, ucTmp);
 			if (ucTmp == 0) {
-				pFont->ucFontstyle &= ~FONT_UNDERLINE;
+				pFont->usFontStyle &= ~FONT_UNDERLINE;
 			} else {
-				pFont->ucFontstyle |= FONT_UNDERLINE;
+				pFont->usFontStyle |= FONT_UNDERLINE;
 			}
 		}
 	}
@@ -722,16 +893,18 @@ bGet1PicInfo(int iFodo,
 	UCHAR	aucChpx[12];
 
 	fail(iFodo < 0 || aucGrpprl == NULL || pPicture == NULL);
-	fail(tBytes > sizeof(aucChpx));
 
-	DBG_PRINT_BLOCK(aucGrpprl + iFodo, tBytes);
+	if (tBytes > sizeof(aucChpx)) {
+		NO_DBG_PRINT_BLOCK(aucGrpprl + iFodo, tBytes);
+		tBytes = sizeof(aucChpx);
+	}
 
 	/* Build the CHPX structure */
 	(void)memset(aucChpx, 0, sizeof(aucChpx));
 	(void)memcpy(aucChpx, aucGrpprl + iFodo, min(tBytes, sizeof(aucChpx)));
 
 	ulTmp = ulGetLong(8, aucChpx);
-	if (ulTmp != 0) {
+	if (ulTmp != 0 && ulTmp < MAX_FILESIZE) {
 		pPicture->ulPictureOffset = ulTmp;
 		DBG_HEX(pPicture->ulPictureOffset);
 		return TRUE;
@@ -751,16 +924,18 @@ bGet2PicInfo(int iFodo,
 	UCHAR	aucChpx[18];
 
 	fail(iFodo < 0 || aucGrpprl == NULL || pPicture == NULL);
-	fail(tBytes > sizeof(aucChpx));
 
-	DBG_PRINT_BLOCK(aucGrpprl + iFodo, tBytes);
+	if (tBytes > sizeof(aucChpx)) {
+		NO_DBG_PRINT_BLOCK(aucGrpprl + iFodo, tBytes);
+		tBytes = sizeof(aucChpx);
+	}
 
 	/* Build the CHPX structure */
 	(void)memset(aucChpx, 0, sizeof(aucChpx));
 	(void)memcpy(aucChpx, aucGrpprl + iFodo, min(tBytes, sizeof(aucChpx)));
 
 	ulTmp = ulGetLong(14, aucChpx);
-	if (ulTmp != 0) {
+	if (ulTmp != 0 && ulTmp < MAX_FILESIZE) {
 		pPicture->ulPictureOffset = ulTmp;
 		DBG_HEX(pPicture->ulPictureOffset);
 		DBG_DEC(tBytes);
@@ -796,6 +971,7 @@ vGet2ChrInfo(FILE *pFile, int iWordVersion, const UCHAR *aucHeader)
 	DBG_DEC(tCharInfoLen);
 	if (tCharInfoLen < 4) {
 		DBG_DEC(tCharInfoLen);
+		return;
 	}
 
 	aucBuffer = xmalloc(tCharInfoLen);
@@ -806,8 +982,7 @@ vGet2ChrInfo(FILE *pFile, int iWordVersion, const UCHAR *aucHeader)
 	NO_DBG_PRINT_BLOCK(aucBuffer, tCharInfoLen);
 
 	tLen = (tCharInfoLen - 4) / 6;
-	tSize = tLen * sizeof(USHORT);
-	ausCharPage = xmalloc(tSize);
+	ausCharPage = xcalloc(tLen, sizeof(USHORT));
 	for (iIndex = 0, tOffset = (tLen + 1) * 4;
 	     iIndex < (int)tLen;
 	     iIndex++, tOffset += 2) {
@@ -845,6 +1020,9 @@ vGet2ChrInfo(FILE *pFile, int iWordVersion, const UCHAR *aucHeader)
 		iRun = (int)ucGetByte(0x1ff, aucFpage);
 		NO_DBG_DEC(iRun);
 		for (iIndex2 = 0; iIndex2 < iRun; iIndex2++) {
+			if ((iRun + 1) * 4 + iIndex2 >= BIG_BLOCK_SIZE) {
+				break;
+			}
 			ulCharPos = ulGetLong(iIndex2 * 4, aucFpage);
 			ulFileOffset = ulCharPos;
 			iFodo = 2 * (int)ucGetByte(
